@@ -33,8 +33,11 @@ from gi.repository import Gst
 from gi.repository import GstController
 from gi.repository import Gtk
 
+from pitivi.check import MISSING_SOFT_DEPS
 from pitivi.clip_properties.alignment import AlignmentEditor
 from pitivi.clip_properties.color import ColorProperties
+from pitivi.clip_properties.compositing import CompositingProperties
+from pitivi.clip_properties.markers import ClipMarkersProperties
 from pitivi.clip_properties.title import TitleProperties
 from pitivi.configure import get_pixmap_dir
 from pitivi.configure import get_ui_dir
@@ -42,8 +45,10 @@ from pitivi.configure import in_devel
 from pitivi.effects import EffectsPopover
 from pitivi.effects import EffectsPropertiesManager
 from pitivi.effects import HIDDEN_EFFECTS
+from pitivi.trackerperspective import CoverObjectPopover
 from pitivi.undo.timeline import CommitTimelineFinalizingAction
-from pitivi.utils.custom_effect_widgets import setup_custom_effect_widgets
+from pitivi.utils.custom_effect_widgets import create_custom_prop_widget_cb
+from pitivi.utils.custom_effect_widgets import create_custom_widget_cb
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import disconnect_all_by_func
 from pitivi.utils.pipeline import PipelineError
@@ -64,9 +69,12 @@ from pitivi.utils.ui import SPACING
 DEFAULT_TEXT = _("Title Clip")
 FOREGROUND_DEFAULT_COLOR = 0xFFFFFFFF  # White
 BACKGROUND_DEFAULT_COLOR = 0x00000000  # Transparent
+OUTLINE_DEFAULT_COLOR = 0xFF000000  # Black
 DEFAULT_FONT_DESCRIPTION = "Sans 36"
 DEFAULT_VALIGNMENT = "absolute"
 DEFAULT_HALIGNMENT = "absolute"
+DEFAULT_DROP_SHADOW = True
+DEFAULT_BLENDING = "over"
 
 # Max speed rate we allow to be applied to clips.
 # The minimum is 1 / MAX_RATE.
@@ -92,7 +100,7 @@ class ClipProperties(Gtk.ScrolledWindow, Loggable):
         viewport.show()
         self.add(viewport)
 
-        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=SPACING)
         vbox.show()
         viewport.add(vbox)
 
@@ -118,10 +126,18 @@ class ClipProperties(Gtk.ScrolledWindow, Loggable):
         self.color_expander.set_vexpand(False)
         vbox.pack_start(self.color_expander, False, False, 0)
 
+        self.compositing_expander = CompositingProperties(app)
+        self.compositing_expander.set_vexpand(False)
+        vbox.pack_start(self.compositing_expander, False, False, 0)
+
         self.effect_expander = EffectProperties(app)
         self.effect_expander.set_vexpand(False)
         vbox.pack_start(self.effect_expander, False, False, 0)
 
+        self.marker_expander = ClipMarkersProperties(app)
+        self.marker_expander.set_vexpand(False)
+        vbox.pack_start(self.marker_expander, False, False, 0)
+
         self.helper_box = self.create_helper_box()
         self.clips_box.pack_start(self.helper_box, False, False, 0)
 
@@ -131,7 +147,9 @@ class ClipProperties(Gtk.ScrolledWindow, Loggable):
         self.speed_expander.set_clip(None)
         self.title_expander.set_source(None)
         self.color_expander.set_source(None)
+        self.compositing_expander.set_source(None)
         self.effect_expander.set_clip(None)
+        self.marker_expander.set_clip(None)
 
         self._project = None
         self._selection = None
@@ -146,15 +164,15 @@ class ClipProperties(Gtk.ScrolledWindow, Loggable):
         label.set_xalign(0)
         box.pack_start(label, False, False, SPACING)
 
-        title_button = Gtk.Button()
-        title_button.set_label(_("Create a title clip"))
-        title_button.connect("clicked", self.create_title_clip_cb)
-        box.pack_start(title_button, False, False, SPACING)
+        self._title_button = Gtk.Button()
+        self._title_button.set_label(_("Create a title clip"))
+        self._title_button.connect("clicked", self.create_title_clip_cb)
+        box.pack_start(self._title_button, False, False, SPACING)
 
-        color_button = Gtk.Button()
-        color_button.set_label(_("Create a color clip"))
-        color_button.connect("clicked", self.create_color_clip_cb)
-        box.pack_start(color_button, False, False, SPACING)
+        self._color_button = Gtk.Button()
+        self._color_button.set_label(_("Create a color clip"))
+        self._color_button.connect("clicked", self.create_color_clip_cb)
+        box.pack_start(self._color_button, False, False, SPACING)
 
         box.show_all()
         return box
@@ -174,9 +192,11 @@ class ClipProperties(Gtk.ScrolledWindow, Loggable):
             properties = {"text": DEFAULT_TEXT,
                           "foreground-color": BACKGROUND_DEFAULT_COLOR,
                           "color": FOREGROUND_DEFAULT_COLOR,
+                          "outline-color": OUTLINE_DEFAULT_COLOR,
                           "font-desc": DEFAULT_FONT_DESCRIPTION,
                           "valignment": DEFAULT_VALIGNMENT,
-                          "halignment": DEFAULT_HALIGNMENT}
+                          "halignment": DEFAULT_HALIGNMENT,
+                          "draw-shadow": DEFAULT_DROP_SHADOW}
             for prop, value in properties.items():
                 res = source.set_child_property(prop, value)
                 assert res, prop
@@ -204,17 +224,13 @@ class ClipProperties(Gtk.ScrolledWindow, Loggable):
         self._project = project
 
     def _selection_changed_cb(self, selection):
-        selected_clips = selection.selected
-        single_clip_selected = len(selected_clips) == 1
-        self.helper_box.set_visible(not single_clip_selected)
+        ges_clip = selection.get_single_clip()
+        self.helper_box.set_visible(not ges_clip)
 
         video_source = None
         title_source = None
         color_clip_source = None
-        ges_clip = None
-        if single_clip_selected:
-            ges_clip = list(selected_clips)[0]
-
+        if ges_clip:
             for child in ges_clip.get_children(False):
                 if isinstance(child, GES.VideoSource):
                     video_source = child
@@ -228,7 +244,9 @@ class ClipProperties(Gtk.ScrolledWindow, Loggable):
         self.speed_expander.set_clip(ges_clip if (not title_source and not color_clip_source) else None)
         self.title_expander.set_source(title_source)
         self.color_expander.set_source(color_clip_source)
+        self.compositing_expander.set_source(video_source)
         self.effect_expander.set_clip(ges_clip)
+        self.marker_expander.set_clip(ges_clip)
 
         self.app.gui.editor.viewer.overlay_stack.select(video_source)
 
@@ -548,16 +566,18 @@ class EffectProperties(Gtk.Expander, Loggable):
 
     def __init__(self, app):
         Gtk.Expander.__init__(self)
+        Loggable.__init__(self)
 
         self.set_expanded(True)
         self.set_label(_("Effects"))
-        Loggable.__init__(self)
 
         self.app = app
         self.clip = None
 
         self.effects_properties_manager = EffectsPropertiesManager(app)
-        setup_custom_effect_widgets(self.effects_properties_manager)
+        # Set up the effects manager to be able to create custom UI.
+        self.effects_properties_manager.connect("create_widget", create_custom_widget_cb)
+        self.effects_properties_manager.connect("create_property_widget", create_custom_prop_widget_cb)
 
         self.drag_lines_pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(
             os.path.join(get_pixmap_dir(), "grip-lines-solid.svg"),
@@ -579,11 +599,21 @@ class EffectProperties(Gtk.Expander, Loggable):
         self.add_effect_button.set_popover(self.effect_popover)
         self.add_effect_button.props.halign = Gtk.Align.CENTER
 
+        self.object_tracker_box = Gtk.ButtonBox()
+        self.object_tracker_box.props.halign = Gtk.Align.CENTER
+
+        self.cover_popover: Optional[Gtk.Popover] = None
+        self.cover_object_button: Optional[Gtk.MenuButton] = None
+        if "cvtracker" not in MISSING_SOFT_DEPS:
+            self.cover_object_button = Gtk.MenuButton(_("Cover Object"))
+            self.object_tracker_box.pack_start(self.cover_object_button, False, False, 0)
+
         self.drag_dest_set(Gtk.DestDefaults.DROP, [EFFECT_TARGET_ENTRY],
                            Gdk.DragAction.COPY)
 
         self.expander_box.pack_start(self.effects_listbox, False, False, 0)
         self.expander_box.pack_start(self.add_effect_button, False, False, PADDING)
+        self.expander_box.pack_start(self.object_tracker_box, False, False, PADDING)
 
         self.add(self.expander_box)
 
@@ -592,34 +622,36 @@ class EffectProperties(Gtk.Expander, Loggable):
         self.connect("drag-leave", self._drag_leave_cb)
         self.connect("drag-data-received", self._drag_data_received_cb)
 
-        self.add_effect_button.connect("toggled", self._add_effect_button_cb)
+        self.add_effect_button.connect("toggled", self._add_effect_button_toggled_cb)
+        if self.cover_object_button:
+            self.cover_object_button.connect("toggled", self._cover_object_button_toggled_cb)
 
         self.show_all()
 
-    def _add_effect_button_cb(self, button):
+    def _add_effect_button_toggled_cb(self, button):
         # MenuButton interacts directly with the popover, bypassing our subclassed method
         if button.props.active:
             self.effect_popover.search_entry.set_text("")
 
+    def _cover_object_button_toggled_cb(self, button):
+        if button.props.active:
+            self.cover_popover.update_object_list()
+
     def _create_effect_row(self, effect):
         if is_time_effect(effect):
             return None
 
-        effect_info = self.app.effects.get_info(effect.props.bin_description)
-
         vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
 
-        row_drag_icon = Gtk.Image.new_from_pixbuf(self.drag_lines_pixbuf)
-
         toggle = Gtk.CheckButton()
         toggle.props.active = effect.props.active
 
+        effect_info = self.app.effects.get_info(effect)
         effect_label = Gtk.Label(effect_info.human_name)
         effect_label.set_tooltip_text(effect_info.description)
 
         # Set up revealer + expander
-        effect_config_ui = self.effects_properties_manager.get_effect_configuration_ui(
-            effect)
+        effect_config_ui = self.effects_properties_manager.get_effect_configuration_ui(effect)
         config_ui_revealer = Gtk.Revealer()
         config_ui_revealer.add(effect_config_ui)
 
@@ -633,9 +665,10 @@ class EffectProperties(Gtk.Expander, Loggable):
 
         remove_effect_button = Gtk.Button.new_from_icon_name("window-close",
                                                              Gtk.IconSize.BUTTON)
-        remove_effect_button.props.margin_right = PADDING
+        remove_effect_button.props.margin_end = PADDING
 
         row_widgets_box = Gtk.Box()
+        row_drag_icon = Gtk.Image.new_from_pixbuf(self.drag_lines_pixbuf)
         row_widgets_box.pack_start(row_drag_icon, False, False, PADDING)
         row_widgets_box.pack_start(toggle, False, False, PADDING)
         row_widgets_box.pack_start(expander, True, True, PADDING)
@@ -737,6 +770,7 @@ class EffectProperties(Gtk.Expander, Loggable):
 
         self.clip = clip
         if self.clip:
+            cover_object_button_show = False
             self.clip.connect("child-added", self._track_element_added_cb)
             self.clip.connect("child-removed", self._track_element_removed_cb)
             for track_element in self.clip.get_children(recursive=True):
@@ -744,11 +778,16 @@ class EffectProperties(Gtk.Expander, Loggable):
                     if is_time_effect(track_element):
                         continue
                     self._connect_to_track_element(track_element)
-
+                if isinstance(track_element, GES.VideoUriSource) and not clip.asset.is_image():
+                    cover_object_button_show = True
+            if self.cover_object_button:
+                self.cover_object_button.set_visible(cover_object_button_show)
+                if cover_object_button_show:
+                    self.cover_popover = CoverObjectPopover(self.app, self.clip)
+                    self.cover_object_button.set_popover(self.cover_popover)
             self._update_listbox()
-            self.show()
-        else:
-            self.hide()
+
+        self.props.visible = bool(self.clip)
 
     def _track_element_added_cb(self, unused_clip, track_element):
         if isinstance(track_element, GES.BaseEffect):
@@ -797,7 +836,7 @@ class EffectProperties(Gtk.Expander, Loggable):
 
     def _drag_data_get_cb(self, eventbox, drag_context, selection_data, unused_info, unused_timestamp):
         row = eventbox.get_parent()
-        effect_info = self.app.effects.get_info(row.effect.props.bin_description)
+        effect_info = self.app.effects.get_info(row.effect)
         effect_name = effect_info.human_name
 
         data = bytes(effect_name, "UTF-8")
@@ -866,10 +905,7 @@ class EffectProperties(Gtk.Expander, Loggable):
 
     def _move_effect(self, clip, source_index, drop_index):
         # Handle edge cases
-        if drop_index < 0:
-            drop_index = 0
-        if drop_index > len(clip.get_top_effects()) - 1:
-            drop_index = len(clip.get_top_effects()) - 1
+        drop_index = min(max(0, drop_index), len(clip.get_top_effects()) - 1)
         if source_index == drop_index:
             # Noop.
             return
@@ -901,6 +937,7 @@ class TransformationProperties(Gtk.Expander, Loggable):
         self.spin_buttons_handler_ids = {}
         self.set_label(_("Transformation"))
         self.set_expanded(True)
+        self._aspect_ratio: Optional[Gst.Fraction] = None
 
         self.builder = Gtk.Builder()
         self.builder.add_from_file(os.path.join(get_ui_dir(),
@@ -946,6 +983,14 @@ class TransformationProperties(Gtk.Expander, Loggable):
             self.__set_prop("posx", x)
             self.__set_prop("posy", y)
 
+    def _update_aspect_ratio_button_image(self):
+        image = self.builder.get_object("aspect_ratio_image")
+        if self._aspect_ratio is not None:
+            icon_name = "chain-connected-symbolic"
+        else:
+            icon_name = "chain-broken-symbolic"
+        image.props.icon_name = icon_name
+
     def _init_buttons(self):
         clear_button = self.builder.get_object("clear_button")
         clear_button.connect("clicked", self._default_values_cb)
@@ -973,6 +1018,10 @@ class TransformationProperties(Gtk.Expander, Loggable):
         self.__setup_spin_button("width_spinbtn", "width")
         self.__setup_spin_button("height_spinbtn", "height")
 
+        self.aspect_ratio_button = self.builder.get_object("aspect_ratio_button")
+        self.aspect_ratio_button.connect("clicked", self._aspect_ratio_button_clicked_cb)
+        self._update_aspect_ratio_button_image()
+
     def __get_keyframes_timestamps(self):
         keyframes_ts = []
         for prop in ["posx", "posy", "width", "height"]:
@@ -1019,12 +1068,12 @@ class TransformationProperties(Gtk.Expander, Loggable):
             else:
                 self._activate_keyframes_btn.set_tooltip_text(
                     _("Activate keyframes"))
-            self.source.ui_element.show_default_keyframes()
+            self.source.ui.show_default_keyframes()
         else:
             self._prev_keyframe_btn.set_sensitive(True)
             self._next_keyframe_btn.set_sensitive(True)
             self._activate_keyframes_btn.set_tooltip_text(_("Hide keyframes"))
-            self.source.ui_element.show_multiple_keyframes(
+            self.source.ui.show_multiple_keyframes(
                 list(self.__control_bindings.values()))
 
     def __update_control_bindings(self):
@@ -1072,9 +1121,10 @@ class TransformationProperties(Gtk.Expander, Loggable):
                 control_source = GstController.InterpolationControlSource()
                 control_source.props.mode = GstController.InterpolationMode.LINEAR
                 self.__own_bindings_change = True
-                self.source.set_control_source(
-                    control_source, prop, "direct-absolute")
-                self.__own_bindings_change = False
+                try:
+                    self.source.set_control_source(control_source, prop, "direct-absolute")
+                finally:
+                    self.__own_bindings_change = False
                 self.__set_default_keyframes_values(control_source, prop)
 
                 binding = self.source.get_control_binding(prop)
@@ -1192,12 +1242,22 @@ class TransformationProperties(Gtk.Expander, Loggable):
         self.spin_buttons[property_name] = spinbtn
         self.spin_buttons_handler_ids[property_name] = handler_id
 
+    def _aspect_ratio_button_clicked_cb(self, aspect_ratio_button):
+        if self._aspect_ratio is None:
+            res, width = self.__get_source_property("width")
+            assert res
+            res, height = self.__get_source_property("height")
+            assert res
+            self._aspect_ratio = Gst.Fraction(width, height)
+        else:
+            self._aspect_ratio = None
+        self._update_aspect_ratio_button_image()
+
     def _on_value_changed_cb(self, spinbtn, prop):
         if not self.source:
             return
 
-        value = spinbtn.get_value()
-
+        value = int(spinbtn.get_value())
         res, cvalue = self.__get_source_property(prop)
         if not res:
             return
@@ -1207,6 +1267,15 @@ class TransformationProperties(Gtk.Expander, Loggable):
                                              finalizing_action=CommitTimelineFinalizingAction(self._project.pipeline),
                                              toplevel=True):
                 self.__set_prop(prop, value)
+                if self._aspect_ratio is not None:
+                    if prop == "width":
+                        fraction = value / self._aspect_ratio
+                        height = int(fraction.num / fraction.denom)
+                        self.__set_prop("height", height)
+                    if prop == "height":
+                        fraction = value * self._aspect_ratio
+                        width = int(fraction.num / fraction.denom)
+                        self.__set_prop("width", width)
             self.app.gui.editor.viewer.overlay_stack.update(self.source)
 
     def set_source(self, source):
