@@ -616,6 +616,7 @@ class AssetPreviewer(Previewer, Loggable):
             # bringing the pipeline back to PAUSED.
             self.pipeline.set_state(Gst.State.PAUSED)
             return
+
         pipeline = Gst.parse_launch(
             "uridecodebin uri={uri} name=decode ! "
             "videoconvert ! "
@@ -684,7 +685,10 @@ class AssetPreviewer(Previewer, Loggable):
 
         if not self.thumb_width:
             self.debug("Finding thumb width")
+            # The pipeline will call `_update_thumbnails` after it sets
+            # the missing `thumb_width`.
             self._setup_pipeline()
+            # There is nothing else we can do now without `thummb_width`.
             return False
 
         # Update the thumbnails with what we already have, if anything.
@@ -790,7 +794,7 @@ class AssetPreviewer(Previewer, Loggable):
         self.debug("Waiting for UI to become idle for: %s",
                    path_from_uri(self.uri))
         self.__start_id = GLib.idle_add(self._start_thumbnailing_cb,
-                                        priority=GLib.PRIORITY_LOW)
+                                        priority=GLib.PRIORITY_DEFAULT_IDLE + 50)
 
     def stop_generation(self):
         if self.__start_id:
@@ -839,7 +843,7 @@ class VideoPreviewer(Gtk.Layout, AssetPreviewer, Zoomable):
 
         self.get_style_context().add_class("VideoPreviewer")
 
-        self.ges_elem = ges_elem
+        self.ges_elem: GES.VideoUriSource = ges_elem
         self.thumbs = {}
 
         # Connect signals and fire things up
@@ -865,21 +869,22 @@ class VideoPreviewer(Gtk.Layout, AssetPreviewer, Zoomable):
 
     def _update_thumbnails(self):
         """Updates the thumbnail widgets for the clip at the current zoom."""
+        # The thumb_width is available after the pipeline has been started.
         if not self.thumb_width or not self.ges_elem.get_track() or not self.ges_elem.props.active:
-            # The thumb_width will be available when pipeline has been started
             return
 
         thumbs = {}
         queue = []
-        interval = self.thumb_interval(self.thumb_width)
 
-        y = (self.props.height_request - self.thumb_height) / 2
+        interval = self.thumb_interval(self.thumb_width)
+        y = (self.props.height_request - self.thumb_height) // 2
         clip = self.ges_elem.get_parent()
-        for position in range(clip.props.start, clip.props.start + clip.props.duration, interval):
-            x = Zoomable.ns_to_pixel(position)
+        for element_position in range(0, self.ges_elem.props.duration, interval):
+            x = Zoomable.ns_to_pixel(element_position)
 
             # Convert position in the timeline to the internal position in the source element
-            position = clip.get_internal_time_from_timeline_time(self.ges_elem, position)
+            internal_position = clip.get_internal_time_from_timeline_time(self.ges_elem, self.ges_elem.props.start + element_position)
+            position = quantize(internal_position, interval)
             try:
                 thumb = self.thumbs.pop(position)
                 self.move(thumb, x, y)
@@ -895,9 +900,11 @@ class VideoPreviewer(Gtk.Layout, AssetPreviewer, Zoomable):
             else:
                 if position not in self.failures and position != self.position:
                     queue.append(position)
+
         for thumb in self.thumbs.values():
             self.remove(thumb)
         self.thumbs = thumbs
+
         self.queue = queue
         if queue:
             self.become_controlled()
@@ -922,7 +929,10 @@ class VideoPreviewer(Gtk.Layout, AssetPreviewer, Zoomable):
 
     def _inpoint_changed_cb(self, unused_ges_timeline_element, unused_param_spec):
         """Handles the changing of the in-point of the clip."""
-        self._update_thumbnails()
+        # Whenever the inpoint changes, the duration also changes, as we never
+        # "roll". We rely on the handler for the duration change event to update
+        # the thumbnails.
+        self.debug("Inpoint change ignored, expecting following duration change")
 
     def _duration_changed_cb(self, unused_ges_timeline_element, unused_param_spec):
         """Handles the changing of the duration of the clip."""
@@ -1164,9 +1174,9 @@ class AudioPreviewer(Gtk.Layout, Previewer, Zoomable, Loggable):
         self.surface = None
         # The zoom level when self.surface has been created.
         self._surface_zoom_level = 0
-        # The samples range used when self.surface has been created.
-        self._surface_start_ns = 0
-        self._surface_end_ns = 0
+        # The pixels range self.surface corresponds to.
+        self._surface_start_px = 0
+        self._surface_end_px = 0
         # The playback rate from last time the surface was updated.
         self._rate = 1.0
 
@@ -1278,18 +1288,27 @@ class AudioPreviewer(Gtk.Layout, Previewer, Zoomable, Loggable):
             # Nothing to draw.
             return
 
-        # The area we have to refresh is determined by the start and end
-        # calculated in the context of the asset duration.
-        rect = Gdk.cairo_get_clip_rectangle(context)[1]
-        clip = self.ges_elem.get_parent()
+        # The area we have to refresh is this rect inside the clip.
+        # For example rect.x is > 0 when the start of the clip is out of view.
+        # rect.width = how many pixels of the clip are in view horizontally.
+        res, rect = Gdk.cairo_get_clip_rectangle(context)
+        assert res
+
+        start = self.ges_elem.props.start
         inpoint = self.ges_elem.props.in_point
         duration = self.ges_elem.props.duration
-        max_duration = self.ges_elem.get_asset().get_filesource_asset().get_duration()
-        start_ns = min(max(0, self.pixel_to_ns(rect.x) + inpoint), max_duration)
-        end_ns = min(max(0, self.pixel_to_ns(rect.x + rect.width) + inpoint), max_duration)
+
         # Get the overall rate of the clip in the current area the clip is used
-        # FIXME: Smarted computation will be needed when we make the rate keyframeable
-        rate = (duration - inpoint) / clip.get_timeline_time_from_internal_time(self.ges_elem, duration)
+        clip = self.ges_elem.get_parent()
+        internal_end = clip.get_internal_time_from_timeline_time(self.ges_elem, start + duration)
+        internal_duration = internal_end - inpoint
+        rate = internal_duration / duration
+
+        inpoint_px = self.ns_to_pixel(start) - self.ns_to_pixel(start - inpoint / rate)
+        max_duration_px = self.ns_to_pixel(clip.maxduration)
+
+        start_px = min(max(0, inpoint_px + rect.x), max_duration_px)
+        end_px = min(max(0, inpoint_px + rect.x + rect.width), max_duration_px)
 
         zoom = self.get_current_zoom_level()
         height = self.get_allocation().height - 2 * CLIP_BORDER_WIDTH
@@ -1297,25 +1316,26 @@ class AudioPreviewer(Gtk.Layout, Previewer, Zoomable, Loggable):
         if not self.surface or \
                 height != self.surface.get_height() or \
                 zoom != self._surface_zoom_level or \
-                start_ns < self._surface_start_ns or \
+                start_px < self._surface_start_px or \
                 rate != self._rate or \
-                end_ns > self._surface_end_ns:
+                end_px > self._surface_end_px:
+            # Generate a new surface since the previously generated one, if any,
+            # cannot be reused.
             if self.surface:
                 self.surface.finish()
                 self.surface = None
             self._surface_zoom_level = zoom
             # The generated waveform is for an extended range if possible,
             # so if the user scrolls we don't rebuild the waveform every time.
-            extra = self.pixel_to_ns(WAVEFORM_SURFACE_EXTRA_PX)
-            self._surface_start_ns = max(0, start_ns - extra)
-            self._surface_end_ns = min(end_ns + extra, max_duration)
+            self._surface_start_px = max(0, start_px - WAVEFORM_SURFACE_EXTRA_PX)
             self._rate = rate
+            self._surface_end_px = min(end_px + WAVEFORM_SURFACE_EXTRA_PX, max_duration_px)
 
             sample_duration = SAMPLE_DURATION / rate
-            range_start = min(max(0, int(self._surface_start_ns / sample_duration)), len(self.samples))
-            range_end = min(max(0, int(self._surface_end_ns / sample_duration)), len(self.samples))
+            range_start = min(max(0, int(self.pixel_to_ns(self._surface_start_px) / sample_duration)), len(self.samples))
+            range_end = min(max(0, int(self.pixel_to_ns(self._surface_end_px) / sample_duration)), len(self.samples))
             samples = self.samples[range_start:range_end]
-            surface_width = self.ns_to_pixel(self._surface_end_ns - self._surface_start_ns)
+            surface_width = self._surface_end_px - self._surface_start_px
             self.surface = renderer.fill_surface(samples, surface_width, height)
 
         # Paint the surface, ignoring the clipped rect.
@@ -1324,8 +1344,8 @@ class AudioPreviewer(Gtk.Layout, Previewer, Zoomable, Loggable):
         # the surface in context, if the entire asset would be drawn.
         # 2. - inpoint, because we're drawing a clip, not the entire asset.
         context.set_operator(cairo.OPERATOR_OVER)
-        offset = self.ns_to_pixel(self._surface_start_ns - inpoint)
-        context.set_source_surface(self.surface, offset, CLIP_BORDER_WIDTH)
+        offset_px = self._surface_start_px - inpoint_px
+        context.set_source_surface(self.surface, offset_px, CLIP_BORDER_WIDTH)
         context.paint()
 
     def _emit_done_on_idle(self):
@@ -1388,7 +1408,15 @@ class TitlePreviewer(Gtk.Layout, Previewer, Zoomable, Loggable):
             self.queue_draw()
 
     def do_draw(self, context):
-        rect = Gdk.cairo_get_clip_rectangle(context)[1]
+        width = self.get_allocated_width()
+        height = self.get_allocated_height()
+
+        # The rect that needs to be drawn
+        exists, rect = Gdk.cairo_get_clip_rectangle(context)
+        if not exists:
+            return
+
+        # Text color white
         context.set_source_rgb(1, 1, 1)
 
         # Get text
@@ -1400,47 +1428,52 @@ class TitlePreviewer(Gtk.Layout, Previewer, Zoomable, Loggable):
 
         # Adapt to RTL/LTR direction
         direction = Pango.unichar_direction(escaped_text[0])
-        if direction in (Pango.Direction.LTR, Pango.Direction.NEUTRAL):
-            stops = (0, 1)
-            x_pos = 10
-            grad = cairo.LinearGradient(rect.width * 0.66, 0,
-                                        rect.width * 0.91, 0)
-        else:
-            stops = (1, 0)
-            x_pos = -10
-            grad = cairo.LinearGradient(rect.width * 0.09, 0,
-                                        rect.width * 0.34, 0)
+        ltr = direction in (Pango.Direction.LTR, Pango.Direction.NEUTRAL)
 
-        # Gradient to make text "fade out"
+        x_pos = 10 if ltr else -10
+        y_pos = int((height / 2) - 11)
+        # Draw the text only if it intersects the rectangle to be drawn.
+        if rect.y + rect.height > y_pos:
+            # Setup Pango layout for drawing the text.
+            layout = PangoCairo.create_layout(context)
+            layout.set_markup(escaped_text, -1)
+
+            layout.set_auto_dir(True)
+            layout.set_font_description(self._font_desc)
+            layout.set_width(width * Pango.SCALE)
+
+            # Prevent lines from being wrapped
+            layout.set_ellipsize(Pango.EllipsizeMode.END)
+
+            # Draw text
+            context.move_to(x_pos, y_pos)
+            PangoCairo.show_layout(context, layout)
+
+        # Draw gradient on top of text to make the text "fade out".
+        # The gradient is mostly transparent and turning opaque at the very end.
+        if ltr:
+            x0 = width * 0.66
+            x1 = width * 0.91
+        else:
+            x0 = width * 0.09
+            x1 = width * 0.34
+        # Control vector of the gradient is x0 -> x1.
+        grad = cairo.LinearGradient(x0, 0, x1, 0)
         if self._selected:
             color = (0.14, 0.133, 0.15)
         else:
             color = (0.368, 0.305, 0.4)
-
-        grad.add_color_stop_rgba(stops[0], color[0], color[1], color[2], 0)
-        grad.add_color_stop_rgba(stops[1], color[0], color[1], color[2], 1)
-
-        # Setup Pango layout
-        layout = PangoCairo.create_layout(context)
-        layout.set_auto_dir(True)
-        layout.set_font_description(self._font_desc)
-        layout.set_width(rect.width * Pango.SCALE)
-
-        # Prevent lines from being wrapped
-        layout.set_ellipsize(Pango.EllipsizeMode.END)
-
-        # Draw text
-        layout.set_markup(escaped_text, -1)
-        context.move_to(x_pos, (rect.height / 2) - 11)
-        PangoCairo.show_layout(context, layout)
-
-        # Draw gradient
+        # Set the start offset of the control vector to transparent (if ltr).
+        grad.add_color_stop_rgba(0.0, color[0], color[1], color[2], 0 if ltr else 1)
+        # Set the end offset of the control vector to opaque (if ltr).
+        grad.add_color_stop_rgba(1.0, color[0], color[1], color[2], 1 if ltr else 0)
+        # The rectangle to be filled represents the entire surface.
         context.rectangle(0, 0, rect.width, rect.height)
         context.set_source(grad)
         context.fill()
 
-    def set_selected(self, select):
-        self._selected = select
+    def set_selected(self, selected):
+        self._selected = selected
 
     def release(self):
         # Nothing to release
