@@ -14,9 +14,7 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 from gettext import gettext as _
 
 import cairo
@@ -36,9 +34,9 @@ from pitivi.utils.ui import time_to_string
 
 
 # Tuples of:
-# - an interval lengths in seconds for which a timestamp will be displayed
+# - an interval duration in seconds for which a timestamp will be displayed
 # - how the ticks should be displayed for this interval:
-#   (count per interval, height ratio) tuples.
+#   (tick interval duration in seconds, height ratio) tuples.
 SCALES = (
     (0.1, ((0.1, 1.0), (0.05, .5), (0.01, .25))),
     (0.2, ((0.2, 1.0), (0.1, .5), (0.05, .25))),
@@ -82,28 +80,30 @@ NORMAL_FONT_SIZE = 13
 SMALL_FONT_SIZE = 11
 
 
-class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
+class ScaleRuler(Gtk.DrawingArea, Loggable):
     """Widget for displaying the ruler.
 
     Displays a series of consecutive intervals. For each interval its beginning
     time is shown. If zoomed in enough, shows the frames in alternate colors.
 
     Attributes:
-        timeline (TimelineContainer): The timeline container used to handle
-            scroll events.
-        _pipeline (Pipeline): The pipeline of the project.
+        zoom (pitivi.utils.timeline.Zoomable): Zoom controller.
+        settings (pitivi.settings.GlobalSettings): The settings of the app.
+        style_context (Gtk.StyleContext): The style context for drawing.
+        _pipeline (pitivi.utils.pipeline.Pipeline): The pipeline of the project.
     """
 
-    def __init__(self, timeline):
+    def __init__(self, zoom, settings, style_context):
         Gtk.DrawingArea.__init__(self)
-        Zoomable.__init__(self)
         Loggable.__init__(self)
         self.log("Creating new ScaleRuler")
 
-        self.timeline = timeline
+        self.zoom = zoom
+        self.settings = settings
+        self.style_context = style_context
         self._pipeline = None
-        hadj = timeline.timeline.hadj
-        hadj.connect("value-changed", self._hadj_value_changed_cb)
+        self.ges_timeline = None
+
         self.add_events(Gdk.EventMask.POINTER_MOTION_MASK |
                         Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK |
                         Gdk.EventMask.SCROLL_MASK)
@@ -115,31 +115,46 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         self.pixbuf_offset_painted = 0
 
         self.position = 0  # In nanoseconds
-        self.frame_rate = Gst.Fraction(1 / 1)
-        self.ns_per_frame = float(1 / self.frame_rate) * Gst.SECOND
 
-        self.scales = SCALES
+        self._scales = [(interval, list(reversed(ticks))) for interval, ticks in SCALES]
 
-    def _hadj_value_changed_cb(self, hadj):
-        """Handles the adjustment value change."""
-        self.pixbuf_offset = hadj.get_value()
-        self.queue_draw()
+        # Update colors when theme or color preferences change.
+        Gtk.Settings.get_default().connect("notify::gtk-theme-name", self._update_colors_cb)
+        Gtk.Settings.get_default().connect("notify::gtk-application-prefer-dark-theme", self._update_colors_cb)
 
-# Zoomable interface override
-
-    def zoomChanged(self):
-        self.queue_draw()
-
-# Timeline position changed method
-
-    def setPipeline(self, pipeline):
+    def set_pipeline(self, pipeline):
         self._pipeline = pipeline
-        self._pipeline.connect('position', self.timelinePositionCb)
+        self.ges_timeline = pipeline.props.timeline
+        self._pipeline.connect("position", self._pipeline_position_cb)
 
-    def timelinePositionCb(self, unused_pipeline, position):
+    def _pipeline_position_cb(self, unused_pipeline, position):
         self.position = position
         self.queue_draw()
 
+    def _set_colors(self):
+        # pylint: disable=attribute-defined-outside-init
+        context = self.get_style_context()
+        color_normal = gtk_style_context_get_color(context, Gtk.StateFlags.NORMAL)
+        color_insensitive = gtk_style_context_get_color(context, Gtk.StateFlags.BACKDROP)
+        self._color_normal = color_normal
+        self._color_dimmed = Gdk.RGBA(
+            *[(x * 3 + y * 2) / 5
+              for x, y in ((color_normal.red, color_insensitive.red),
+                           (color_normal.green, color_insensitive.green),
+                           (color_normal.blue, color_insensitive.blue))])
+
+        self._color_subtle = Gdk.RGBA(
+            *[(x * 3 + y * 2) / 10
+              for x, y in ((color_normal.red, color_insensitive.red),
+                           (color_normal.green, color_insensitive.green),
+                           (color_normal.blue, color_insensitive.blue))])
+
+        # Two colors with high contrast.
+        self._color_frame = gtk_style_context_get_color(context, Gtk.StateFlags.LINK)
+
+    def _update_colors_cb(self, settings, gparams):
+        self._set_colors()
+
 # Gtk.Widget overrides
 
     def do_configure_event(self, unused_event):
@@ -155,34 +170,21 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         # Create a new buffer
         self.pixbuf = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
 
-        context = self.app.gui.get_style_context()
-
-        color_normal = gtk_style_context_get_color(context, Gtk.StateFlags.NORMAL)
-        color_insensitive = gtk_style_context_get_color(context, Gtk.StateFlags.BACKDROP)
-        self._color_normal = color_normal
-        self._color_dimmed = Gdk.RGBA(
-            *[(x * 3 + y * 2) / 5
-              for x, y in ((color_normal.red, color_insensitive.red),
-                           (color_normal.green, color_insensitive.green),
-                           (color_normal.blue, color_insensitive.blue))])
-
-        # Two colors with high contrast.
-        self._color_frame = gtk_style_context_get_color(context, Gtk.StateFlags.LINK)
-
+        self._set_colors()
         return False
 
     def do_draw(self, context):
         if self.pixbuf is None:
-            self.info('No buffer to paint')
+            self.info("No buffer to paint")
             return False
 
         pixbuf = self.pixbuf
 
         # Draw on a temporary context and then copy everything.
         drawing_context = cairo.Context(pixbuf)
-        self.drawBackground(drawing_context)
-        self.drawRuler(drawing_context)
-        self.drawPosition(drawing_context)
+        self.draw_background(drawing_context)
+        self.draw_ruler(drawing_context)
+        self.draw_position(drawing_context)
         pixbuf.flush()
 
         context.set_source_surface(self.pixbuf, 0.0, 0.0)
@@ -195,19 +197,18 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
             return False
 
         button = event.button
-        if button == 3 or (button == 1 and self.app.settings.leftClickAlsoSeeks):
+        if button == 3 or (button == 1 and self.settings.leftClickAlsoSeeks):
             self.debug("button pressed at x:%d", event.x)
-            position = self.pixelToNs(event.x + self.pixbuf_offset)
+            position = self.pixel_to_ns(event.x + self.pixbuf_offset)
             self._pipeline.simple_seek(position)
             self.__set_tooltip_text(position, True)
         return False
 
     def do_button_release_event(self, event):
         button = event.button
-        if button == 3 or (button == 1 and self.app.settings.leftClickAlsoSeeks):
+        if button == 3 or (button == 1 and self.settings.leftClickAlsoSeeks):
             self.debug("button released at x:%d", event.x)
-            self.app.gui.focusTimeline()
-            position = self.pixelToNs(event.x + self.pixbuf_offset)
+            position = self.pixel_to_ns(event.x + self.pixbuf_offset)
             self.__set_tooltip_text(position)
         return False
 
@@ -215,10 +216,10 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         if not self._pipeline:
             return False
 
-        position = self.pixelToNs(event.x + self.pixbuf_offset)
+        position = self.pixel_to_ns(event.x + self.pixbuf_offset)
 
         seek_mask = Gdk.ModifierType.BUTTON3_MASK
-        if self.app.settings.leftClickAlsoSeeks:
+        if self.settings.leftClickAlsoSeeks:
             seek_mask |= Gdk.ModifierType.BUTTON1_MASK
 
         seeking = event.state & seek_mask
@@ -229,69 +230,63 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
 
         return False
 
-    def do_scroll_event(self, event):
-        self.timeline.timeline.do_scroll_event(event)
-
-    def setProjectFrameRate(self, rate):
-        """Sets the lowest scale based on the specified project framerate."""
-        self.frame_rate = rate
-        self.ns_per_frame = float(Gst.SECOND / self.frame_rate)
-        self.scales = (2 / rate, 5 / rate, 10 / rate) + SCALES
-
     def __set_tooltip_text(self, position, seeking=False):
         """Updates the tooltip."""
+        position = max(0, position)
         if seeking:
-            timeline_duration = self.timeline.ges_timeline.props.duration
-            if position > timeline_duration:
-                position = timeline_duration
+            timeline_duration = self.ges_timeline.props.duration
+            position = min(position, timeline_duration)
         human_time = beautify_length(position)
-        cur_frame = int(position / self.ns_per_frame) + 1
+        cur_frame = self.ges_timeline.get_frame_at(position) + 1
         self.set_tooltip_text(human_time + "\n" + _("Frame #%d") % cur_frame)
 
 # Drawing methods
 
-    def drawBackground(self, context):
+    def draw_background(self, context):
         width = context.get_target().get_width()
         height = context.get_target().get_height()
-        style_context = self.app.gui.get_style_context()
-        Gtk.render_background(style_context, context, 0, 0, width, height)
+        Gtk.render_background(self.style_context, context, 0, 0, width, height)
 
-    def drawRuler(self, context):
+    def draw_ruler(self, context):
         context.set_font_face(NORMAL_FONT)
         context.set_font_size(NORMAL_FONT_SIZE)
 
-        spacing, interval_seconds, ticks = self._getSpacing(context)
+        spacing, interval_seconds, ticks = self._get_spacing(context)
         offset = self.pixbuf_offset % spacing
-        self.drawFrameBoundaries(context)
-        self.drawTicks(context, offset, spacing, interval_seconds, ticks)
-        self.drawTimes(context, offset, spacing, interval_seconds)
+        self.draw_frame_boundaries(context)
+        self.draw_ticks(context, offset, spacing, interval_seconds, ticks)
+        self.draw_times(context, offset, spacing, interval_seconds)
 
-    def _getSpacing(self, context):
+    def _get_spacing(self, context):
         # The longest timestamp we display is 0:00:00 because
         # when we display millis, they are displayed by themselves.
         min_interval_width = context.text_extents("0:00:00")[2] * 1.3
-        zoom = Zoomable.zoomratio
-        for interval_seconds, ticks in SCALES:
-            interval_width = interval_seconds * zoom
+        zoomratio = self.zoom.zoomratio
+        for interval_seconds, ticks in self._scales:
+            interval_width = interval_seconds * zoomratio
             if interval_width >= min_interval_width:
                 return interval_width, interval_seconds, ticks
         raise Exception(
             "Failed to find an interval size for textwidth:%s, zoomratio:%s" %
-            (min_interval_width, Zoomable.zoomratio))
+            (min_interval_width, zoomratio))
 
-    def drawTicks(self, context, offset, spacing, interval_seconds, ticks):
+    def draw_ticks(self, context, offset, spacing, interval_seconds, ticks):
         for tick_interval, height_ratio in ticks:
             count_per_interval = interval_seconds / tick_interval
             space = spacing / count_per_interval
             if space < MIN_TICK_SPACING_PIXELS:
-                break
+                continue
             paintpos = 0.5 - offset
-            set_cairo_color(context, self._color_normal)
+
+            color = (self._color_normal if height_ratio == 1
+                     else self._color_subtle)
+
+            set_cairo_color(context, color)
             while paintpos < context.get_target().get_width():
-                self._drawTick(context, paintpos, height_ratio)
+                self._draw_tick(context, paintpos, height_ratio)
                 paintpos += space
 
-    def _drawTick(self, context, paintpos, height_ratio):
+    def _draw_tick(self, context, paintpos, height_ratio):
         # We need to use 0.5 pixel offsets to get a sharp 1 px line in cairo
         paintpos = int(paintpos - 0.5) + 0.5
         target_height = context.get_target().get_height()
@@ -302,10 +297,10 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         context.close_path()
         context.stroke()
 
-    def drawTimes(self, context, offset, spacing, interval_seconds):
+    def draw_times(self, context, offset, spacing, interval_seconds):
         # figure out what the optimal offset is
         interval = int(Gst.SECOND * interval_seconds)
-        current_time = self.pixelToNs(self.pixbuf_offset)
+        current_time = self.zoom.pixel_to_ns(self.pixbuf_offset)
         paintpos = TIMES_LEFT_MARGIN_PIXELS
         if offset > 0:
             current_time = current_time - (current_time % interval) + interval
@@ -326,12 +321,12 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
             context.move_to(int(paintpos), 1 - y_bearing)
             current = split(time_to_string(int(current_time)))
             millis = current_time % Gst.SECOND > 0
-            self._drawTime(context, current, previous, millis)
+            self._draw_time(context, current, previous, millis)
             previous = current
             paintpos += spacing
             current_time += interval
 
-    def _drawTime(self, context, current, previous, millis):
+    def _draw_time(self, context, current, previous, millis):
         hour = int(current[0])
         for index, (element, previous_element) in enumerate(zip(current, previous)):
             if index <= 1 and not hour:
@@ -358,7 +353,7 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
             if small:
                 context.set_font_size(NORMAL_FONT_SIZE)
 
-    def drawFrameBoundaries(self, context):
+    def draw_frame_boundaries(self, context):
         """Draws the alternating rectangles that represent the project frames.
 
         These are drawn only at high zoom levels.
@@ -366,21 +361,23 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         These are based on the project's framerate settings, not the actual
         frames on the assets.
         """
-        frame_width = self.nsToPixel(self.ns_per_frame)
-        if not frame_width >= FRAME_MIN_WIDTH_PIXELS:
+        if not self.ges_timeline:
+            # Timeline not set yet
+            return
+
+        frame_width = self.zoom.ns_to_pixel(self.ges_timeline.get_frame_time(1))
+        if frame_width < FRAME_MIN_WIDTH_PIXELS:
             return
 
         offset = self.pixbuf_offset % frame_width
         height = context.get_target().get_height()
         y = int(height - FRAME_HEIGHT_PIXELS)
 
-        frame_num = int(
-            self.pixelToNs(self.pixbuf_offset) * float(self.frame_rate) / Gst.SECOND)
+        frame_num = self.ges_timeline.get_frame_at(self.zoom.pixel_to_ns(self.pixbuf_offset))
         paintpos = self.pixbuf_offset - offset
         max_pos = context.get_target().get_width() + self.pixbuf_offset
         while paintpos < max_pos:
-            paintpos = self.nsToPixel(
-                1 / float(self.frame_rate) * Gst.SECOND * frame_num)
+            paintpos = self.zoom.ns_to_pixel(self.ges_timeline.get_frame_time(frame_num))
             if frame_num % 2:
                 set_cairo_color(context, self._color_frame)
                 context.rectangle(
@@ -388,7 +385,7 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
                 context.fill()
             frame_num += 1
 
-    def drawPosition(self, context):
+    def draw_position(self, context):
         """Draws the top part of the playhead.
 
         This should be in sync with the playhead drawn by the timeline.
@@ -402,7 +399,7 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
 
         # Add 0.5 so that the line center is at the middle of the pixel,
         # without this the line appears blurry.
-        xpos = self.nsToPixel(self.position) - self.pixbuf_offset + 0.5
+        xpos = self.zoom.ns_to_pixel(self.position) - self.pixbuf_offset + 0.5
         set_cairo_color(context, PLAYHEAD_COLOR)
 
         context.set_line_width(PLAYHEAD_WIDTH)
@@ -417,3 +414,36 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         context.line_to(xpos - semi_width, y - semi_height)
         context.close_path()
         context.stroke()
+
+
+class TimelineScaleRuler(ScaleRuler, Zoomable):
+    """Widget for displaying a ruler which is connected to a timeline.
+
+    Attributes:
+        timeline_container (TimelineContainer): The timeline container for
+            handling scroll events.
+    """
+
+    def __init__(self, timeline_container):
+        Zoomable.__init__(self)
+        ScaleRuler.__init__(self, self, self.app.settings, self.app.gui.get_style_context())
+        self.log("Creating new TimelineScaleRuler")
+
+        self.timeline_container = timeline_container
+
+        timeline_container.timeline.hadj.connect("value-changed", self._hadj_value_changed_cb)
+
+    def _hadj_value_changed_cb(self, hadj):
+        """Handles the adjustment value change."""
+        self.pixbuf_offset = hadj.get_value()
+        self.queue_draw()
+
+# Zoomable interface override
+
+    def zoom_changed(self):
+        self.queue_draw()
+
+# Gtk.Widget overrides
+
+    def do_scroll_event(self, event):
+        self.timeline_container.timeline.do_scroll_event(event)
