@@ -75,6 +75,14 @@ TOUCH_INPUT_SOURCES = (Gdk.InputSource.TOUCHPAD,
                        Gdk.InputSource.TRACKPOINT,
                        Gdk.InputSource.TABLET_PAD)
 
+CURSORS = {
+    GES.Edge.EDGE_START: Gdk.Cursor.new(Gdk.CursorType.LEFT_SIDE),
+    GES.Edge.EDGE_END: Gdk.Cursor.new(Gdk.CursorType.RIGHT_SIDE)
+}
+
+NORMAL_CURSOR = Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR)
+DRAG_CURSOR = Gdk.Cursor.new(Gdk.CursorType.HAND1)
+
 
 def get_month_format_string():
     """Returns the appropriate format string for month name in time.strftime() function."""
@@ -294,10 +302,40 @@ EDITOR_PERSPECTIVE_CSS = """
         background-image: url('%(marker_hovered)s');
     }
 
+    .ClipMarkersBox {
+        transition: 0.15s ease-out;
+        opacity: 0.7;
+    }
+
+    .ClipMarkersBox:hover {
+        background-color: rgba(0, 0, 0, 0.15);
+        opacity: 0.85;
+    }
+
+    .ClipMarkersBox:selected {
+        background-color: rgb(0, 0, 0);
+        opacity: 1;
+    }
+
+    .ClipMarker {
+        background-image: url('%(clip_marker_unselected)s');
+    }
+
+    .ClipMarker:hover {
+        background-image: url('%(clip_marker_hovered)s');
+    }
+
+    .ClipMarker:selected {
+        background-image: url('%(clip_marker_selected)s');
+    }
+
 """ % ({
     'clip_border_width': CLIP_BORDER_WIDTH,
     'marker_hovered': os.path.join(get_pixmap_dir(), "marker-hover.png"),
     'marker_unselected': os.path.join(get_pixmap_dir(), "marker-unselect.png"),
+    'clip_marker_unselected': os.path.join(get_pixmap_dir(), "clip-marker.png"),
+    'clip_marker_hovered': os.path.join(get_pixmap_dir(), "clip-marker-hover.png"),
+    'clip_marker_selected': os.path.join(get_pixmap_dir(), "clip-marker-select.png"),
     'trimbar_focused': os.path.join(get_pixmap_dir(), "trimbar-focused.png"),
     'trimbar_normal': os.path.join(get_pixmap_dir(), "trimbar-normal.png")})
 
@@ -399,20 +437,18 @@ def gtk_style_context_get_color(context, state):
     return color
 
 
-def argb_to_gdk_rgba(color_int):
-    return Gdk.RGBA(color_int / 256 ** 2 % 256 / 255.,
-                    color_int / 256 ** 1 % 256 / 255.,
-                    color_int / 256 ** 0 % 256 / 255.,
-                    color_int / 256 ** 3 % 256 / 255.)
+def argb_to_gdk_rgba(argb: int) -> Gdk.RGBA:
+    return Gdk.RGBA(((argb >> 16) & 0xFF) / 255,
+                    ((argb >> 8) & 0xFF) / 255,
+                    ((argb >> 0) & 0xFF) / 255,
+                    ((argb >> 24) & 0xFF) / 255)
 
 
-def gdk_rgba_to_argb(color):
-    color_int = 0
-    color_int += int(color.alpha * 255) * 256 ** 3
-    color_int += int(color.red * 255) * 256 ** 2
-    color_int += int(color.green * 255) * 256 ** 1
-    color_int += int(color.blue * 255) * 256 ** 0
-    return color_int
+def gdk_rgba_to_argb(color: Gdk.RGBA) -> int:
+    return ((int(color.alpha * 255) << 24) +
+            (int(color.red * 255) << 16) +
+            (int(color.green * 255) << 8) +
+            int(color.blue * 255))
 
 
 def pack_color_32(red, green, blue, alpha=0xFFFF):
@@ -459,10 +495,6 @@ def unpack_color_64(value):
     return red, green, blue, alpha
 
 
-def hex_to_rgb(value):
-    return tuple(float(int(value[i:i + 2], 16)) / 255.0 for i in range(0, 6, 2))
-
-
 def set_cairo_color(context, color):
     if isinstance(color, Gdk.RGBA):
         cairo_color = (float(color.red), float(color.green), float(color.blue))
@@ -839,20 +871,19 @@ def alter_style_class(style_class, target_widget, css_style):
         css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
 
 
-def set_children_state_recurse(widget, state):
-    widget.set_state_flags(state, False)
-    for child in widget.get_children():
-        child.set_state_flags(state, False)
-        if isinstance(child, Gtk.Container):
-            set_children_state_recurse(child, state)
+def set_state_flags_recurse(widget, state_flags, are_set, ignored_classes=()):
+    """Sets the provided state on all children of the given widget."""
+    if isinstance(widget, ignored_classes):
+        return
 
+    if are_set:
+        widget.set_state_flags(state_flags, clear=False)
+    else:
+        widget.unset_state_flags(state_flags)
 
-def unset_children_state_recurse(widget, state):
-    widget.unset_state_flags(state)
-    for child in widget.get_children():
-        child.unset_state_flags(state)
-        if isinstance(child, Gtk.Container):
-            unset_children_state_recurse(child, state)
+    if isinstance(widget, Gtk.Container):
+        for child in widget.get_children():
+            set_state_flags_recurse(child, state_flags, are_set, ignored_classes)
 
 
 def disable_scroll_event_cb(widget, unused_event):
