@@ -111,14 +111,16 @@ def create_project():
     return project
 
 
-def create_pitivi(**settings):
+def create_pitivi(**settings) -> Pitivi:
+    """Creates a Pitivi app with the specified settings, ready to be tested."""
     app = Pitivi()
     app._setup()
 
+    app.settings = __create_settings(**settings)
+
+    # Patch a main window object so things depending on it can work properly.
     app.gui = mock.Mock()
     app.gui.editor.viewer.action_group = Gio.SimpleActionGroup()
-
-    app.settings = __create_settings(**settings)
     app.gui.editor.editor_state = EditorState(app.project_manager)
 
     return app
@@ -152,13 +154,17 @@ def create_main_loop():
         mainloop.quit()
 
     def run(timeout_seconds=5, until_empty=False):
-        source = GLib.timeout_source_new_seconds(timeout_seconds)
-        source.set_callback(timeout_cb)
-        source.attach()
+        # Limit the test running time only when not debugging.
+        debugging = os.environ.get("PITIVI_VSCODE_DEBUG", False)
+        if not debugging:
+            source = GLib.timeout_source_new_seconds(timeout_seconds)
+            source.set_callback(timeout_cb)
+            source.attach()
         if until_empty:
-            GLib.idle_add(mainloop.quit)
+            GLib.idle_add(mainloop.quit, priority=GLib.PRIORITY_LOW + 1)
         GLib.MainLoop.run(mainloop)
-        source.destroy()
+        if not debugging:
+            source.destroy()
         if timed_out:
             raise Exception("Timed out after %s seconds" % timeout_seconds)
 
@@ -262,9 +268,14 @@ def setup_project_with_clips(assets_names: List[str]):
     def decorator(func):
         nonlocal assets_names
 
-        @setup_project(assets_names)
+        @setup_project(list(set(assets_names)))
         def wrapper(self):
             assets = self.app.project_manager.current_project.list_assets(GES.UriClip)
+
+            # Insert the assets in the same order that we got them.
+            assets_by_name = {os.path.basename(asset.get_id()): asset for asset in assets}
+            assets = [assets_by_name[name] for name in assets_names]
+
             self.timeline_container.insert_assets(assets, self.timeline.props.duration)
 
             func(self)
@@ -282,13 +293,12 @@ def setup_timeline(func):
         self.layer = self.timeline.append_layer()
         self.action_log = self.app.action_log
 
-        project = self.app.project_manager.current_project
         self.timeline_container = TimelineContainer(self.app, editor_state=self.app.gui.editor.editor_state)
-        self.timeline_container.set_project(project)
+        self.timeline_container.set_project(self.project)
         self.app.gui.editor.timeline_ui = self.timeline_container
 
         timeline = self.timeline_container.timeline
-        timeline.app.project_manager.current_project = project
+        self.assertEqual(timeline.app.project_manager.current_project, self.project)
         timeline.get_parent = mock.MagicMock(return_value=self.timeline_container)
 
         func(self)
@@ -304,6 +314,7 @@ def setup_timeline(func):
 
 
 def setup_clipproperties(func):
+    """Wraps a test, providing a usable ClipProperties."""
     def wrapped(self):
         app = self.timeline_container.app
 
@@ -314,9 +325,14 @@ def setup_clipproperties(func):
         self.transformation_box._new_project_loaded_cb(None, self.project)
 
         self.speed_box = self.clipproperties.speed_expander
+        self.compositing_box = self.clipproperties.compositing_expander
+        self.markers_box = self.clipproperties.marker_expander
 
         func(self)
 
+        del self.markers_box
+        del self.compositing_box
+        del self.speed_box
         del self.transformation_box
         del self.clipproperties
 
@@ -407,12 +423,12 @@ class TestCase(unittest.TestCase, Loggable):
         unused, xges_path = tempfile.mkstemp(suffix=".xges")
         proj_uri = Gst.filename_to_uri(os.path.abspath(xges_path))
 
-        with open(xges_path, "w") as xges_file:
+        with open(xges_path, "w", encoding="UTF-8") as xges_file:
             xges_file.write(xges)
 
         return proj_uri
 
-    def add_clip(self, layer, start, inpoint=0, duration=10, clip_type=GES.TrackType.UNKNOWN):
+    def add_clip(self, layer: GES.Layer, start, inpoint=0, duration=10, clip_type=GES.TrackType.UNKNOWN):
         """Creates a clip on the specified layer."""
         asset = GES.UriClipAsset.request_sync(
             get_sample_uri("tears_of_steel.webm"))
@@ -428,10 +444,13 @@ class TestCase(unittest.TestCase, Loggable):
         self.assertEqual(len(clips), num_clips)
         return clips
 
-    def toggle_clip_selection(self, ges_clip, expect_selected):
-        """Toggles the selection state of @ges_clip."""
-        selected = bool(ges_clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED)
-        self.assertEqual(ges_clip.selected.selected, selected)
+    def click_clip(self, ges_clip: GES.Clip, expect_selected: bool, ctrl_key: bool = False):
+        """Clicks the specified clip."""
+        timeline = ges_clip.ui.timeline
+        original_control_mask = timeline.get_parent().control_mask
+        if ctrl_key:
+            # Pretend the Ctrl key is being pressed.
+            timeline.get_parent().control_mask = True
 
         # Simulate a click on the clip.
         event = mock.Mock(spec=Gdk.EventButton)
@@ -440,18 +459,32 @@ class TestCase(unittest.TestCase, Loggable):
         event.get_button.return_value = (True, 1)
         with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
             get_event_widget.return_value = ges_clip.ui
-            ges_clip.ui.timeline._button_press_event_cb(None, event)
+            timeline._button_press_event_cb(None, event)
             with mock.patch.object(ges_clip.ui, "translate_coordinates") as translate_coordinates:
                 translate_coordinates.return_value = (0, 0)
-                with mock.patch.object(ges_clip.ui.timeline, "get_layer_at") as get_layer_at:
+                with mock.patch.object(timeline, "get_layer_at") as get_layer_at:
                     get_layer_at.return_value = ges_clip.props.layer, None
                     ges_clip.ui._button_release_event_cb(None, event)
-                    ges_clip.ui.timeline._button_release_event_cb(None, event)
+                    timeline._button_release_event_cb(None, event)
 
+        if ctrl_key:
+            timeline.get_parent().control_mask = original_control_mask
+
+        self.assert_clip_selected(ges_clip, expect_selected)
+
+    def assert_clip_selected(self, ges_clip, expect_selected):
         self.assertEqual(bool(ges_clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED),
                          expect_selected)
         self.assertEqual(ges_clip.selected.selected, expect_selected)
 
+        for child in ges_clip.ui.get_children():
+            if not hasattr(child, "selected"):
+                continue
+
+            self.assertEqual(bool(child.get_state_flags() & Gtk.StateFlags.SELECTED),
+                             expect_selected)
+            self.assertEqual(child.selected, expect_selected)
+
     def assert_caps_equal(self, caps1, caps2):
         if isinstance(caps1, str):
             caps1 = Gst.Caps(caps1)
@@ -520,10 +553,12 @@ class TestCase(unittest.TestCase, Loggable):
         self.assertEqual(len(effects), count)
 
     def assert_control_source_values(self, control_source, expected_values, expected_timestamps):
-        values = [timed_value.value for timed_value in control_source.get_all()]
+        keyframes = control_source.get_all()
+
+        values = [timed_value.value for timed_value in keyframes]
         self.assertListEqual(values, expected_values)
 
-        timestamps = [timed_value.timestamp for timed_value in control_source.get_all()]
+        timestamps = [timed_value.timestamp for timed_value in keyframes]
         self.assertListEqual(timestamps, expected_timestamps)
 
     def get_timeline_clips(self):
@@ -540,6 +575,12 @@ class TestCase(unittest.TestCase, Loggable):
                         return element
         return None
 
+    def get_clip_element(self, ges_clip, element_class=GES.VideoSource):
+        for element in ges_clip.get_children(False):
+            if isinstance(element, element_class):
+                return element
+        return None
+
     @staticmethod
     def commit_cb(action_log, stack, stacks):
         stacks.append(stack)
@@ -564,7 +605,7 @@ def created_project_file(asset_uri):
         str: The URI of the new project
     """
     unused_fd, xges_path = tempfile.mkstemp()
-    with open(xges_path, "w") as xges:
+    with open(xges_path, "w", encoding="UTF-8") as xges:
         xges.write("""
 <ges version='0.1'>
   <project>
@@ -594,13 +635,6 @@ def get_sample_uri(sample, samples_dir=None):
     return Gst.filename_to_uri(os.path.join(samples_dir, sample))
 
 
-def get_sample_clip(sample):
-    uri = get_sample_uri(sample)
-    asset = GES.UriClipAsset.request_sync(uri)
-    clip = asset.extract()
-    return clip
-
-
 @contextlib.contextmanager
 def cloned_sample(*samples):
     """Gets a context manager which commits the transaction at the end."""
@@ -618,7 +652,7 @@ def cloned_sample(*samples):
             module["get_sample_uri"] = original_get_sample_uri
 
 
-def get_clip_children(ges_clip, *track_types, recursive=False):
+def get_clip_children(ges_clip: GES.Clip, *track_types: List[GES.TrackType], recursive: bool = False):
     for ges_timeline_element in ges_clip.get_children(recursive):
         if not track_types or ges_timeline_element.get_track_type() in track_types:
             yield ges_timeline_element
