@@ -19,6 +19,7 @@
 import os
 import tempfile
 from unittest import mock
+from unittest import skip
 
 from gi.repository import Gdk
 from gi.repository import GES
@@ -81,8 +82,7 @@ class BaseTestMediaLibrary(common.TestCase):
 
     def _progress_bar_cb(self, progressbar, unused_pspec):
         if self.check_no_transcoding:
-            self.assertTrue(progressbar.props.fraction == 1.0 or
-                            progressbar.props.fraction == 0.0,
+            self.assertTrue(progressbar.props.fraction in (0, 1),
                             "Some transcoding is happening, got progress: %f"
                             % progressbar.props.fraction)
 
@@ -110,7 +110,7 @@ class BaseTestMediaLibrary(common.TestCase):
             "notify::fraction", self._progress_bar_cb)
 
         self._create_assets(samples)
-        self.mainloop.run()
+        self.mainloop.run(timeout_seconds=25)
         self.assertFalse(self.medialibrary._progressbar.props.visible)
 
     def check_add_proxy(self, asset, scaled=False, w=160, h=120,
@@ -240,20 +240,21 @@ class TestMediaLibrary(BaseTestMediaLibrary):
             self.assertEqual(set(project.list_assets(GES.Extractable)), set([target, asset]))
 
             # Remove the asset
-            self.medialibrary.remove_assets_action.emit("activate", None)
+            self.medialibrary.remove_assets_action.activate()
 
             # Make sure that the project has not assets anymore
             self.assertEqual(project.list_assets(GES.Extractable), [])
 
             # Save the project and reload it, making sure there is no asset
             # in that new project
-            project_uri = Gst.filename_to_uri(tempfile.NamedTemporaryFile().name)
-            project.save(project.ges_timeline, project_uri, None, True)
+            with tempfile.NamedTemporaryFile() as temp_file:
+                project_uri = Gst.filename_to_uri(temp_file.name)
+                project.save(project.ges_timeline, project_uri, None, True)
 
-            self._custom_set_up(project_uri)
-            self.assertNotEqual(project, self.app.project_manager.current_project)
-            project = self.app.project_manager.current_project
-            self.assertEqual(project.list_assets(GES.Extractable), [])
+                self._custom_set_up(project_uri)
+                self.assertNotEqual(project, self.app.project_manager.current_project)
+                project = self.app.project_manager.current_project
+                self.assertEqual(project.list_assets(GES.Extractable), [])
 
     def check_selection_post_import(self, **kwargs):
         samples = ["30fps_numeroted_frames_red.mkv",
@@ -497,6 +498,7 @@ class TestMediaLibrary(BaseTestMediaLibrary):
             self.check_import([sample], check_no_transcoding=True,
                               proxying_strategy=ProxyingStrategy.AUTOMATIC)
 
+    @skip("times out")
     def test_import_supported_forced_scaled_audio(self):
         sample = "mp3_sample.mp3"
         with common.cloned_sample(sample):
@@ -520,8 +522,11 @@ class TestMediaLibrary(BaseTestMediaLibrary):
 class TestTaggingAssets(BaseTestMediaLibrary):
 
     def import_assets_in_medialibrary(self):
-        samples = ["30fps_numeroted_frames_red.mkv",
-                   "30fps_numeroted_frames_blue.webm", "1sec_simpsons_trailer.mp4"]
+        samples = [
+            "30fps_numeroted_frames_red.mkv",
+            "30fps_numeroted_frames_blue.webm",
+            "1sec_simpsons_trailer.mp4",
+        ]
         with common.cloned_sample(*samples):
             self.check_import(samples, proxying_strategy=ProxyingStrategy.NOTHING)
 
@@ -557,7 +562,6 @@ class TestTaggingAssets(BaseTestMediaLibrary):
                 raise Exception(tag_name)
 
     def test_adding_tags(self):
-        self.mainloop = common.create_main_loop()
         self.import_assets_in_medialibrary()
         self.medialibrary.flowbox.unselect_all()
 
@@ -612,7 +616,6 @@ class TestTaggingAssets(BaseTestMediaLibrary):
         self.medialibrary.tags_popover.hide()
 
     def test_removing_tags(self):
-        self.mainloop = common.create_main_loop()
         self.import_assets_in_medialibrary()
         tag = "TAG"
         self.add_new_tag(tag)
@@ -667,3 +670,31 @@ class TestTaggingAssets(BaseTestMediaLibrary):
         self.assertEqual(self.medialibrary.witnessed_tags, set())
         for item in self.medialibrary.store:
             self.assertEqual(item.tags, set())
+
+    def check_suggestions(self, text, expected_suggestions):
+        self.medialibrary.search_entry.props.text = text
+        self.medialibrary.search_entry.emit("search-changed")
+
+        suggestions = [item[0] for item in self.medialibrary.search_store]
+        self.assertListEqual(suggestions, expected_suggestions)
+
+    def test_search_suggestions(self):
+        self.import_assets_in_medialibrary()
+
+        self.medialibrary.witnessed_tags = {"red", "blue", "green"}
+
+        self.check_suggestions("", ["tag:blue", "tag:green", "tag:red"])
+        self.check_suggestions("tag:", ["tag:blue", "tag:green", "tag:red"])
+        # Keep in mind that only the suggestions matching the text are shown.
+        # For example at this point, only the "tag:blue" suggestion is shown
+        # because it's the only one that matches the "tag:b" text.
+        self.check_suggestions("tag:b", ["tag:blue", "tag:green", "tag:red"])
+
+        self.check_suggestions("tag:blue ", ["tag:blue tag:green", "tag:blue tag:red"])
+        self.check_suggestions("tag:blue tag:", ["tag:blue tag:green", "tag:blue tag:red"])
+        self.check_suggestions("tag:blue tag:red", ["tag:blue tag:green", "tag:blue tag:red"])
+
+        self.check_suggestions("tag:blue tag:red ", ["tag:blue tag:red tag:green"])
+        self.check_suggestions("tag:blue tag:red word1", ["tag:blue tag:red tag:green"])
+
+        self.check_suggestions("tag:blue tag:red word1 ", ["tag:blue tag:red word1 tag:green"])
