@@ -134,9 +134,9 @@ class TestLayers(common.TestCase):
         self.assertEqual(len(timeline.__on_separators), 1,
                          "The separators must be forgotten only in drag_end()")
 
+    @common.setup_timeline
     def test_media_types(self):
-        timeline_container = common.create_timeline_container()
-        timeline = timeline_container.timeline
+        timeline = self.timeline_container.timeline
 
         ges_layer_1 = timeline.ges_timeline.append_layer()
         ges_layer_2 = timeline.ges_timeline.append_layer()
@@ -241,47 +241,46 @@ class TestGrouping(common.TestCase):
     def test_can_group_ungroup(self):
         timeline_container = common.create_timeline_container()
         timeline = timeline_container.timeline
+        # Timeline empty.
         self.__check_can_group_ungroup(timeline_container, False, False)
+
         ges_clip, = self.add_clips_simple(timeline, 1)
-        self.toggle_clip_selection(ges_clip, expect_selected=True)
+        # No clip selected.
+        self.__check_can_group_ungroup(timeline_container, False, False)
+
+        self.click_clip(ges_clip, expect_selected=True)
+        # An audio-video clip is selected.
         self.__check_can_group_ungroup(timeline_container, False, True)
 
-        timeline_container.ungroup_action.emit("activate", None)
-        self.__check_can_group_ungroup(timeline_container, False, False)
+        timeline_container.ungroup_action.activate()
+        # The resulting audio clip and video clip should both be selected.
+        self.__check_can_group_ungroup(timeline_container, True, False)
 
         layer, = timeline.ges_timeline.get_layers()
         ges_clip0, ges_clip1 = layer.get_clips()
-        self.toggle_clip_selection(ges_clip0, expect_selected=True)
+        self.click_clip(ges_clip0, expect_selected=False, ctrl_key=True)
+        # Only one of the clips remains selected.
         self.__check_can_group_ungroup(timeline_container, False, False)
 
-        # Press <ctrl> so selecting in ADD mode
-        event = mock.Mock()
-        event.keyval = Gdk.KEY_Control_L
-        timeline_container.do_key_press_event(event)
-
-        self.toggle_clip_selection(ges_clip1, expect_selected=True)
+        self.click_clip(ges_clip0, expect_selected=True)
+        self.click_clip(ges_clip1, expect_selected=True, ctrl_key=True)
+        # Both clip are selected.
         self.__check_can_group_ungroup(timeline_container, True, False)
 
-        timeline_container.group_action.emit("activate", None)
+        timeline_container.group_action.activate()
+        # The resulting audio-video clip should be selected.
         self.__check_can_group_ungroup(timeline_container, False, True)
 
     def group_clips(self, timeline_container, clips):
         timeline = timeline_container.timeline
         timeline.app.settings.leftClickAlsoSeeks = False
 
-        # Press <ctrl> so selecting in ADD mode
-        event = mock.Mock()
-        event.keyval = Gdk.KEY_Control_L
-        timeline_container.do_key_press_event(event)
-        timeline.get_clicked_layer_and_pos = mock.Mock()
-        timeline.get_clicked_layer_and_pos.return_value = (None, None)
-
         # Select the 2 clips
         for clip in clips:
             self.assertIsNone(clip.get_parent())
-            self.toggle_clip_selection(clip, expect_selected=True)
+            self.click_clip(clip, expect_selected=True, ctrl_key=True)
 
-        timeline_container.group_action.emit("activate", None)
+        timeline_container.group_action.activate()
 
         for clip in clips:
             # Check that we created a new group
@@ -291,12 +290,13 @@ class TestGrouping(common.TestCase):
                 self.assertEqual(clip.get_toplevel_parent(), selected_clip.get_toplevel_parent())
 
             self.assertEqual(clips[0].get_parent(), clip.get_parent())
-            self.assertTrue(bool(clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED))
-            self.assertTrue(clip.selected)
+            self.assert_clip_selected(clip, expect_selected=True)
 
         group = clips[0].get_parent()
         self.assertEqual(len(group.get_children(False)), len(clips))
 
+        self.assertEqual(len(timeline.selection), len(clips))
+
     def test_group(self):
         timeline_container = common.create_timeline_container()
         timeline = timeline_container.timeline
@@ -309,17 +309,19 @@ class TestGrouping(common.TestCase):
         timeline = timeline_container.timeline
         clips = self.add_clips_simple(timeline, num_clips)
         self.group_clips(timeline_container, clips)
-        layer = timeline.ges_timeline.get_layers()[0]
+        layer, = timeline.ges_timeline.get_layers()
         clips = layer.get_clips()
         self.assertEqual(len(clips), num_clips)
 
-        # Deselect one grouped clip clips
-        self.toggle_clip_selection(clips[0], expect_selected=False)
-
-        # Make sure all the clips have been deselected
+        # Deselect one of the clips in the group.
+        self.click_clip(clips[0], expect_selected=False, ctrl_key=True)
         for clip in clips:
-            self.assertFalse(bool(clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED))
-            self.assertFalse(clip.selected)
+            self.assert_clip_selected(clip, expect_selected=False)
+
+        # Select one of the clips in the group.
+        self.click_clip(clips[0], expect_selected=True)
+        for clip in clips:
+            self.assert_clip_selected(clip, expect_selected=True)
 
     def test_group_ungroup(self):
         num_clips = 2
@@ -328,9 +330,10 @@ class TestGrouping(common.TestCase):
         clips = self.add_clips_simple(timeline, num_clips)
         self.group_clips(timeline_container, clips)
 
-        self.assertEqual(len(timeline.selection.selected), num_clips)
+        # Selecting a clip selects all the clips in its group.
+        self.click_clip(clips[0], expect_selected=True)
 
-        timeline_container.ungroup_action.emit("activate", None)
+        timeline_container.ungroup_action.activate()
         layer = timeline.ges_timeline.get_layers()[0]
         clips = layer.get_clips()
         self.assertEqual(len(clips), num_clips)
@@ -339,51 +342,65 @@ class TestGrouping(common.TestCase):
             self.assertIsNone(clip.get_parent())
 
     def test_group_splitted_clip_and_select_group(self):
-
+        # Create a clip and select it
         timeline_container = common.create_timeline_container()
         timeline = timeline_container.timeline
         clips = self.add_clips_simple(timeline, 1)
         clips[0].props.duration = timeline.ges_timeline.get_frame_time(4)
+        self.click_clip(clips[0], expect_selected=True)
+
+        # Split the clip
         position = timeline.ges_timeline.get_frame_time(2)
-        self.toggle_clip_selection(clips[0], expect_selected=True)
-
         timeline.ges_timeline.get_asset().pipeline.get_position = mock.Mock(return_value=position)
+        timeline_container.split_action.activate()
         layer = timeline.ges_timeline.get_layers()[0]
-
-        # Split
-        timeline_container.split_action.emit("activate", None)
         clips = layer.get_clips()
         self.assertEqual(len(clips), 2)
 
-        # Only the first clip is selected so select the
-        # second one
-        self.assertTrue(clips[0].selected)
-        self.assertFalse(clips[1].selected)
+        # Only the first clip is selected
+        self.assert_clip_selected(clips[0], expect_selected=True)
+        self.assert_clip_selected(clips[1], expect_selected=False)
 
+        # Select the second clip
         event = mock.Mock()
         event.keyval = Gdk.KEY_Control_L
         timeline_container.do_key_press_event(event)
+        self.click_clip(clips[1], expect_selected=True)
         timeline.get_clicked_layer_and_pos = mock.Mock()
         timeline.get_clicked_layer_and_pos.return_value = (None, None)
-        self.toggle_clip_selection(clips[1], expect_selected=True)
         timeline_container.do_key_release_event(event)
 
-        for clip in clips:
-            self.assertTrue(clip.selected)
+        # Both clips are selected
+        self.assert_clip_selected(clips[0], expect_selected=True)
+        self.assert_clip_selected(clips[1], expect_selected=True)
 
-        # Group the two parts
-        timeline_container.group_action.emit("activate", None)
+        # Group the two selected clips
+        timeline_container.group_action.activate()
 
-        self.toggle_clip_selection(clips[1], expect_selected=True)
+        # Deselect the first clip, notice both clips have been deselected
+        self.click_clip(clips[0], expect_selected=False, ctrl_key=True)
+        self.assert_clip_selected(clips[1], expect_selected=False)
+
+        # Select the first clip, notice both clips have been selected
+        self.click_clip(clips[0], expect_selected=True)
+        self.assert_clip_selected(clips[1], expect_selected=True)
+
+        # Deselect the second clip, notice both clips have been deselected
+        self.click_clip(clips[1], expect_selected=False, ctrl_key=True)
+        self.assert_clip_selected(clips[0], expect_selected=False)
+
+        # Select the second clip, notice both clips have been selected
+        self.click_clip(clips[1], expect_selected=True)
+        self.assert_clip_selected(clips[0], expect_selected=True)
 
     def test_ungroup_clip(self):
         timeline_container = common.create_timeline_container()
         timeline = timeline_container.timeline
         ges_clip, = self.add_clips_simple(timeline, 1)
 
-        self.toggle_clip_selection(ges_clip, expect_selected=True)
+        self.click_clip(ges_clip, expect_selected=True)
 
-        timeline_container.ungroup_action.emit("activate", None)
+        timeline_container.ungroup_action.activate()
         layer = timeline.ges_timeline.get_layers()[0]
         ges_clip0, ges_clip1 = layer.get_clips()
 
@@ -449,106 +466,184 @@ class TestGrouping(common.TestCase):
                          "No new layer should have been created")
 
 
-class TestCopyPaste(common.TestCase):
+class TestCutCopyPaste(common.TestCase):
 
-    def copy_clips(self, num_clips):
-        timeline_container = common.create_timeline_container()
-        timeline = timeline_container.timeline
-
-        clips = self.add_clips_simple(timeline, num_clips)
-
-        # Press <ctrl> so selecting in ADD mode
-        event = mock.Mock()
-        event.keyval = Gdk.KEY_Control_L
-        timeline_container.do_key_press_event(event)
-        timeline.get_clicked_layer_and_pos = mock.Mock()
-        timeline.get_clicked_layer_and_pos.return_value = (None, None)
-
-        # Select the 2 clips
+    @common.setup_project_with_clips(assets_names=["mp3_sample.mp3", "tears_of_steel.webm"])
+    def test_cut_paste(self):
+        # Cut the clips.
+        clips = self.layer.get_clips()
         for clip in clips:
-            self.toggle_clip_selection(clip, expect_selected=True)
+            self.click_clip(clip, expect_selected=True, ctrl_key=True)
+        self.timeline_container.cut_action.activate()
 
-        self.assertTrue(timeline_container.copy_action.props.enabled)
-        self.assertFalse(timeline_container.paste_action.props.enabled)
-        timeline_container.copy_action.emit("activate", None)
-        self.assertTrue(timeline_container.paste_action.props.enabled)
+        # Check no clip is present.
+        clips = self.layer.get_clips()
+        self.assertEqual(len(clips), 0)
 
-        return timeline_container
+        # Paste the clips.
+        position = 0
+        self.project.pipeline.get_position = mock.Mock(return_value=position)
+        self.timeline_container.paste_action.activate()
 
-    def test_copy_paste(self):
-        timeline_container = self.copy_clips(2)
-        timeline = timeline_container.timeline
-        layer = timeline.ges_timeline.get_layers()[0]
-        project = timeline.ges_timeline.get_asset()
-
-        clips = layer.get_clips()
+        # Check the clips are pasted at the given position and are selected.
+        clips = self.layer.get_clips()
         self.assertEqual(len(clips), 2)
+        self.assert_clip_selected(clips[0], expect_selected=True)
+        self.assert_clip_selected(clips[1], expect_selected=True)
 
-        # Pasting clips for the first time.
-        position = 20
-        project.pipeline.get_position = mock.Mock(return_value=position)
-        timeline_container.paste_action.emit("activate", None)
+        first_clip_start = clips[0].props.start
+        second_clip_start = clips[1].props.start
+        self.assertEqual(first_clip_start, position)
+        self.assertEqual(second_clip_start, position + clips[0].props.duration)
 
-        n_clips = layer.get_clips()
-        self.assertEqual(len(n_clips), 4)
+        # Cut the first clip.
+        self.click_clip(clips[1], expect_selected=False, ctrl_key=True)
+        # TODO: Remove this line when https://gitlab.gnome.org/GNOME/pitivi/-/issues/2433 is fixed.
+        self.click_clip(clips[0], expect_selected=True)
+        self.timeline_container.cut_action.activate()
 
-        copied_clips = [clip for clip in n_clips if clip not in clips]
+        # Check only the second clip is present.
+        clips = self.layer.get_clips()
+        self.assertEqual(len(clips), 1)
+        self.assert_clip_selected(clips[0], expect_selected=False)
+        self.assertEqual(clips[0].props.start, second_clip_start)
+
+        # Paste the clip again.
+        self.project.pipeline.get_position = mock.Mock(return_value=first_clip_start)
+        self.timeline_container.paste_action.activate()
+
+        # Check both clips are present.
+        clips = self.layer.get_clips()
+        self.assertEqual(len(clips), 2)
+        self.assert_clip_selected(clips[0], expect_selected=True)
+        self.assert_clip_selected(clips[1], expect_selected=False)
+        self.assertEqual(clips[0].props.start, first_clip_start)
+        self.assertEqual(clips[1].props.start, second_clip_start)
+
+    @common.setup_project_with_clips(assets_names=["mp3_sample.mp3", "tears_of_steel.webm"])
+    def test_copy_paste(self):
+        # Copy the clips.
+        clips = self.layer.get_clips()
+        for clip in clips:
+            self.click_clip(clip, expect_selected=True, ctrl_key=True)
+        self.timeline_container.copy_action.activate()
+
+        # Paste the clips for the first time.
+        position = self.project.pipeline.get_duration()
+        self.project.pipeline.get_position = mock.Mock(return_value=position)
+        self.timeline_container.paste_action.activate()
+
+        new_clips = self.layer.get_clips()
+        self.assertEqual(len(new_clips), 4)
+
+        copied_clips = [clip for clip in new_clips if clip not in clips]
         self.assertEqual(len(copied_clips), 2)
         self.assertEqual(copied_clips[0].props.start, position)
-        self.assertEqual(copied_clips[1].props.start, position + 10)
+        self.assertEqual(copied_clips[1].props.start, position + copied_clips[0].props.duration)
 
-        # Pasting same clips second time.
-        position = 40
-        project.pipeline.get_position = mock.Mock(return_value=position)
-        timeline_container.paste_action.emit("activate", None)
+        # Paste the same clips for the second time.
+        position = self.project.pipeline.get_duration()
+        self.project.pipeline.get_position = mock.Mock(return_value=position)
+        self.timeline_container.paste_action.activate()
 
-        n_clips = layer.get_clips()
-        self.assertEqual(len(n_clips), 6)
+        new_clips = self.layer.get_clips()
+        self.assertEqual(len(new_clips), 6)
 
-        copied_clips = [clip for clip in n_clips if clip not in clips]
+        copied_clips = [clip for clip in new_clips if clip not in clips]
         self.assertEqual(len(copied_clips), 4)
         self.assertEqual(copied_clips[2].props.start, position)
-        self.assertEqual(copied_clips[3].props.start, position + 10)
+        self.assertEqual(copied_clips[3].props.start, position + copied_clips[2].props.duration)
 
+    @common.setup_project_with_clips(assets_names=["tears_of_steel.webm"])
     def test_paste_not_possible(self):
-        timeline_container = self.copy_clips(1)
-        timeline = timeline_container.timeline
-        layer = timeline.ges_timeline.get_layers()[0]
-        project = timeline.ges_timeline.get_asset()
-        self.assertEqual(len(layer.get_clips()), 1)
+        # Copy the clip.
+        clip, = self.layer.get_clips()
+        self.click_clip(clip, expect_selected=True)
+        self.timeline_container.copy_action.activate()
 
+        # Try to paste the clip at the same position.
         position = 0
-        project.pipeline.get_position = mock.Mock(return_value=position)
-        timeline_container.paste_action.emit("activate", None)
-        self.assertEqual(len(layer.get_clips()), 1)
+        self.project.pipeline.get_position = mock.Mock(return_value=position)
+        self.timeline_container.paste_action.activate()
 
-    def test_paste_selection(self):
-        timeline_container = self.copy_clips(1)
-        timeline = timeline_container.timeline
-        layer = timeline.ges_timeline.get_layers()[0]
-        project = timeline.ges_timeline.get_asset()
-
-        # Paste a single clip.
-        clips = layer.get_clips()
+        clips = self.layer.get_clips()
         self.assertEqual(len(clips), 1)
-        position = 10
-        project.pipeline.get_position = mock.Mock(return_value=position)
-        timeline_container.paste_action.emit("activate", None)
 
-        clips = layer.get_clips()
-        self.assertListEqual([bool(clip.selected) for clip in clips], [False, True])
+    @common.setup_project_with_clips(assets_names=["tears_of_steel.webm"])
+    def test_paste_selection(self):
+        # Copy the clip.
+        clip, = self.layer.get_clips()
+        self.click_clip(clip, expect_selected=True)
+        self.timeline_container.copy_action.activate()
 
-        # The copy_clips() above simulates CTRL button press, so this will add 1st clip to the selection.
-        self.toggle_clip_selection(clips[0], True)
-        timeline_container.copy_action.emit("activate", None)
+        # Paste the clip.
+        position = self.project.pipeline.get_duration()
+        self.project.pipeline.get_position = mock.Mock(return_value=position)
+        self.timeline_container.paste_action.activate()
 
-        # Paste two clips.
-        position = 20
-        project.pipeline.get_position = mock.Mock(return_value=position)
-        timeline_container.paste_action.emit("activate", None)
+        clips = self.layer.get_clips()
+        self.assert_clip_selected(clips[0], expect_selected=False)
+        self.assert_clip_selected(clips[1], expect_selected=True)
 
-        clips = layer.get_clips()
-        self.assertListEqual([bool(clip.selected) for clip in clips], [False, False, True, True])
+        # Add the first clip to the selection.
+        self.click_clip(clips[0], expect_selected=True, ctrl_key=True)
+        self.timeline_container.copy_action.activate()
+
+        # Paste both clips.
+        position = self.project.pipeline.get_duration()
+        self.project.pipeline.get_position = mock.Mock(return_value=position)
+        self.timeline_container.paste_action.activate()
+
+        clips = self.layer.get_clips()
+        self.assert_clip_selected(clips[0], expect_selected=False)
+        self.assert_clip_selected(clips[1], expect_selected=False)
+        self.assert_clip_selected(clips[2], expect_selected=True)
+        self.assert_clip_selected(clips[3], expect_selected=True)
+
+    @common.setup_project_with_clips(assets_names=["tears_of_steel.webm"])
+    def test_actions_enabled_status(self):
+        # Check status after the initialization.
+        self.assertFalse(self.timeline_container.cut_action.props.enabled)
+        self.assertFalse(self.timeline_container.copy_action.props.enabled)
+        self.assertFalse(self.timeline_container.paste_action.props.enabled)
+
+        # Check status after selecting the clip.
+        clip, = self.layer.get_clips()
+        self.click_clip(clip, expect_selected=True)
+
+        self.assertTrue(self.timeline_container.cut_action.props.enabled)
+        self.assertTrue(self.timeline_container.copy_action.props.enabled)
+        self.assertFalse(self.timeline_container.paste_action.props.enabled)
+
+        # Check status after copying the clip.
+        self.timeline_container.copy_action.activate()
+
+        self.assertTrue(self.timeline_container.cut_action.props.enabled)
+        self.assertTrue(self.timeline_container.copy_action.props.enabled)
+        self.assertTrue(self.timeline_container.paste_action.props.enabled)
+
+        # Check status after cutting the clip.
+        self.timeline_container.cut_action.activate()
+
+        self.assertFalse(self.timeline_container.cut_action.props.enabled)
+        self.assertFalse(self.timeline_container.copy_action.props.enabled)
+        self.assertTrue(self.timeline_container.paste_action.props.enabled)
+
+        # Check status after pasting the clip.
+        position = 0
+        self.project.pipeline.get_position = mock.Mock(return_value=position)
+        self.timeline_container.paste_action.activate()
+
+        self.assertTrue(self.timeline_container.cut_action.props.enabled)
+        self.assertTrue(self.timeline_container.copy_action.props.enabled)
+        self.assertTrue(self.timeline_container.paste_action.props.enabled)
+
+        # Check status after deleting the clip.
+        self.timeline_container.delete_action.activate()
+
+        self.assertFalse(self.timeline_container.cut_action.props.enabled)
+        self.assertFalse(self.timeline_container.copy_action.props.enabled)
+        self.assertTrue(self.timeline_container.paste_action.props.enabled)
 
 
 class TestEditing(common.TestCase):
@@ -588,7 +683,7 @@ class TestEditing(common.TestCase):
                          "No new layer should have been created")
 
 
-class TestShiftSelection(common.TestCase):
+class TestClipsSelection(common.TestCase):
 
     def __reset_clips_selection(self, timeline):
         """Unselects all clips in the timeline."""
@@ -600,9 +695,9 @@ class TestShiftSelection(common.TestCase):
 
     def __check_selected(self, selected_clips, not_selected_clips):
         for clip in selected_clips:
-            self.assertEqual(clip.selected._selected, True)
+            self.assert_clip_selected(clip, expect_selected=True)
         for clip in not_selected_clips:
-            self.assertEqual(clip.selected._selected, False)
+            self.assert_clip_selected(clip, expect_selected=False)
 
     def __check_simple(self, left_click_also_seeks):
         timeline_container = common.create_timeline_container()
@@ -755,6 +850,36 @@ class TestShiftSelection(common.TestCase):
         self.__check_shift_selection_multiple_layers(left_click_also_seeks=False)
         self.__check_shift_selection_multiple_layers(left_click_also_seeks=True)
 
+    def test_clip_unselection(self):
+        """Tests whether the clips are unselected properly."""
+        timeline = common.create_timeline_container().timeline
+
+        clip1, clip2 = self.add_clips_simple(timeline, 2)
+        self.click_clip(clip1, expect_selected=True, ctrl_key=True)
+        self.click_clip(clip2, expect_selected=True, ctrl_key=True)
+        self.__check_selected([clip1, clip2], [])
+
+        # Unselect clip2.
+        self.click_clip(clip2, expect_selected=False, ctrl_key=True)
+        self.__check_selected([clip1], [clip2])
+
+    def test_grouped_clips_unselection(self):
+        """Tests grouped clips are unselected together."""
+        timeline_container = common.create_timeline_container()
+        timeline = timeline_container.timeline
+
+        clip1, clip2, clip3 = self.add_clips_simple(timeline, 3)
+        self.click_clip(clip1, expect_selected=True, ctrl_key=True)
+        self.click_clip(clip2, expect_selected=True, ctrl_key=True)
+        timeline_container.group_action.activate()
+
+        self.click_clip(clip3, expect_selected=True, ctrl_key=True)
+        self.__check_selected([clip1, clip2, clip3], [])
+
+        # Unselect the group.
+        self.click_clip(clip1, expect_selected=False, ctrl_key=True)
+        self.__check_selected([clip3], [clip1, clip2])
+
 
 class TestTimelineContainer(common.TestCase):
     """Tests for the TimelineContainer class."""
@@ -882,13 +1007,13 @@ class TestKeyboardShiftClips(common.TestCase):
         event.keyval = Gdk.KEY_Control_L
         self.timeline_container.do_key_press_event(event)
         for clip in ges_clips:
-            self.toggle_clip_selection(clip, expect_selected=True)
+            self.click_clip(clip, expect_selected=True)
         self.timeline_container.do_key_release_event(event)
 
-        self.timeline_container.shift_forward_action.emit("activate", None)
+        self.timeline_container.shift_forward_action.activate()
         self.assertListEqual([clip.start - delta for clip in ges_clips], clip_original_starts)
 
-        self.timeline_container.shift_backward_action.emit("activate", None)
+        self.timeline_container.shift_backward_action.activate()
         self.assertListEqual([clip.start for clip in ges_clips], clip_original_starts)
 
     @common.setup_project(["tears_of_steel.webm"])
@@ -921,24 +1046,19 @@ class TestKeyboardShiftClips(common.TestCase):
         ges_clip3 = self.add_clip(self.layer, 13 * Gst.SECOND, duration=2 * Gst.SECOND)
         ges_clip4 = self.add_clip(self.layer, 15 * Gst.SECOND, duration=2 * Gst.SECOND)
 
-        event = mock.Mock()
-        event.keyval = Gdk.KEY_Control_L
-        self.timeline_container.do_key_press_event(event)
-        self.toggle_clip_selection(ges_clip1, expect_selected=True)
-        self.toggle_clip_selection(ges_clip2, expect_selected=True)
-        self.toggle_clip_selection(ges_clip3, expect_selected=True)
-        self.toggle_clip_selection(ges_clip4, expect_selected=True)
+        self.click_clip(ges_clip1, expect_selected=True, ctrl_key=True)
+        self.click_clip(ges_clip2, expect_selected=True, ctrl_key=True)
+        self.click_clip(ges_clip3, expect_selected=True, ctrl_key=True)
+        self.click_clip(ges_clip4, expect_selected=True, ctrl_key=True)
 
-        self.timeline_container.do_key_release_event(event)
-
-        self.timeline_container.shift_forward_action.emit("activate", None)
+        self.timeline_container.shift_forward_action.activate()
 
         self.assertEqual(5 * Gst.SECOND, ges_clip1.start)
         self.assertEqual(10 * Gst.SECOND, ges_clip2.start)
         self.assertEqual(13 * Gst.SECOND, ges_clip3.start)
         self.assertEqual(15 * Gst.SECOND, ges_clip4.start)
 
-        self.timeline_container.shift_backward_action.emit("activate", None)
+        self.timeline_container.shift_backward_action.activate()
 
         self.assertEqual(5 * Gst.SECOND, ges_clip1.start)
         self.assertEqual(10 * Gst.SECOND, ges_clip2.start)
@@ -954,12 +1074,12 @@ class TestSnapClips(common.TestCase):
         ges_clip1 = self.add_clip(self.layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
         ges_clip2 = self.add_clip(self.layer, 9 * Gst.SECOND, duration=2 * Gst.SECOND)
 
-        self.toggle_clip_selection(ges_clip1, expect_selected=True)
+        self.click_clip(ges_clip1, expect_selected=True)
 
-        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.timeline_container.snap_clips_forward_action.activate()
         self.assertEqual(ges_clip1.start, ges_clip2.start - ges_clip1.duration)
 
-        self.timeline_container.snap_clips_backward_action.emit("activate", None)
+        self.timeline_container.snap_clips_backward_action.activate()
         self.assertEqual(ges_clip1.start, 0)
 
     @common.setup_timeline
@@ -969,17 +1089,14 @@ class TestSnapClips(common.TestCase):
         ges_clip2 = self.add_clip(self.layer, 7 * Gst.SECOND, duration=2 * Gst.SECOND)
         ges_clip3 = self.add_clip(self.layer, 11 * Gst.SECOND, duration=2 * Gst.SECOND)
 
-        event = mock.Mock()
-        event.keyval = Gdk.KEY_Control_L
-        self.timeline_container.do_key_press_event(event)
-        self.toggle_clip_selection(ges_clip1, expect_selected=True)
-        self.toggle_clip_selection(ges_clip2, expect_selected=True)
+        self.click_clip(ges_clip1, expect_selected=True, ctrl_key=True)
+        self.click_clip(ges_clip2, expect_selected=True, ctrl_key=True)
 
-        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.timeline_container.snap_clips_forward_action.activate()
         self.assertEqual(ges_clip2.start, ges_clip3.start - ges_clip2.duration)
         self.assertEqual(ges_clip1.start, ges_clip2.start - ges_clip1.duration)
 
-        self.timeline_container.snap_clips_backward_action.emit("activate", None)
+        self.timeline_container.snap_clips_backward_action.activate()
         self.assertEqual(ges_clip1.start, 0)
         self.assertEqual(ges_clip2.start, ges_clip1.start + ges_clip1.duration)
 
@@ -993,12 +1110,12 @@ class TestSnapClips(common.TestCase):
         clip = self.add_clip(layer2, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
         end_clip = self.add_clip(layer2, 30 * Gst.SECOND, duration=2 * Gst.SECOND)
 
-        self.toggle_clip_selection(clip, expect_selected=True)
+        self.click_clip(clip, expect_selected=True)
 
-        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.timeline_container.snap_clips_forward_action.activate()
         self.assertEqual(clip.start, end_clip.start - clip.duration)
 
-        self.timeline_container.snap_clips_backward_action.emit("activate", None)
+        self.timeline_container.snap_clips_backward_action.activate()
         self.assertEqual(clip.start, 0)
 
     @common.setup_timeline
@@ -1007,9 +1124,9 @@ class TestSnapClips(common.TestCase):
         ges_clip = self.add_clip(self.layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
         clip_start = ges_clip.start
 
-        self.toggle_clip_selection(ges_clip, expect_selected=True)
+        self.click_clip(ges_clip, expect_selected=True)
 
-        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.timeline_container.snap_clips_forward_action.activate()
         self.assertEqual(ges_clip.start, clip_start)
 
     @common.setup_timeline
@@ -1019,7 +1136,7 @@ class TestSnapClips(common.TestCase):
         self.add_clip(self.layer, 30 * Gst.SECOND, duration=2 * Gst.SECOND)
         ges_clip = self.add_clip(layer2, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
 
-        self.toggle_clip_selection(ges_clip, expect_selected=True)
+        self.click_clip(ges_clip, expect_selected=True)
 
-        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.timeline_container.snap_clips_forward_action.activate()
         self.assertEqual(ges_clip.start, self.timeline.get_duration() - ges_clip.duration)
