@@ -26,11 +26,13 @@ from gi.repository import GstController
 from gi.repository import Gtk
 
 from pitivi.timeline.layer import Layer
+from pitivi.undo.base import PropertyChangedAction
 from pitivi.undo.project import AssetAddedAction
 from pitivi.undo.timeline import ClipAdded
 from pitivi.undo.timeline import ClipRemoved
+from pitivi.undo.timeline import CommitTimelineFinalizingAction
 from pitivi.undo.timeline import TrackElementAdded
-from pitivi.undo.undo import PropertyChangedAction
+from pitivi.utils.timeline import EditingContext
 from pitivi.utils.ui import LAYER_HEIGHT
 from pitivi.utils.ui import URI_TARGET_ENTRY
 from tests import common
@@ -50,10 +52,7 @@ class TestSelectionResetWhenRemovingClip(common.TestCase):
     def check_selection(self, *expected_selected_clips):
         self.assertSetEqual(set(self.timeline_container.timeline.selection), set(expected_selected_clips))
         for clip in self.get_timeline_clips():
-            if clip in expected_selected_clips:
-                self.assertTrue(clip.selected)
-            else:
-                self.assertFalse(clip.selected)
+            self.assert_clip_selected(clip, expect_selected=clip in expected_selected_clips)
 
     @common.setup_timeline
     def test_redo_delete_when_selected(self):
@@ -157,14 +156,13 @@ class TestTimelineObserver(common.TestCase):
 
         self.assertTrue(self.layer.add_clip(clip1))
         self.assertTrue(self.layer.add_clip(clip2))
-        # The selection does not care about GES.Groups, only about GES.Clips.
-        self.timeline_container.timeline.selection.select([clip1, clip2])
         self.assertEqual(clip1.props.timeline, self.layer.get_timeline())
         self.assertEqual(clip2.props.timeline, self.layer.get_timeline())
 
+        self.timeline_container.timeline.selection.select([clip1, clip2])
         self.timeline_container.group_action.activate(None)
-        self.assertTrue(isinstance(clip1.get_parent(), GES.Group))
         self.assertEqual(clip1.get_parent(), clip2.get_parent())
+        self.assertTrue(isinstance(clip1.get_parent(), GES.Group), type(clip1.get_parent()))
 
         self.timeline_container.ungroup_action.activate(None)
         self.assertIsNone(clip1.get_parent())
@@ -203,7 +201,7 @@ class TestTimelineObserver(common.TestCase):
         self.assertEqual(len(clips), 1, clips)
         self.assertEqual(len(clips[0].get_children(False)), 2)
 
-        common.TestCase.toggle_clip_selection(self, clips[0], True)
+        self.click_clip(clips[0], expect_selected=True)
         self.timeline_container.ungroup_action.activate(None)
         clips = list(self.get_timeline_clips())
         self.assertEqual(len(clips), 2, clips)
@@ -342,7 +340,7 @@ class TestLayerObserver(common.TestCase):
             self.layer.add_clip(clip1)
 
         stack = self.action_log.undo_stacks[0]
-        self.assertEqual(len(stack.done_actions), 7, stack.done_actions)
+        self.assertEqual(len(stack.done_actions), 9, stack.done_actions)
         self.assertTrue(isinstance(stack.done_actions[0], ClipAdded))
         self.assertTrue(clip1 in self.get_timeline_clips())
 
@@ -383,7 +381,8 @@ class TestLayerObserver(common.TestCase):
         clip = GES.TitleClip()
         self.layer.add_clip(clip)
 
-        self.timeline_container.add_layer_action.emit("activate", None)
+        self.timeline_container.update_actions()
+        self.timeline_container.add_layer_action.activate()
 
         layers = self.timeline.get_layers()
         self.assertEqual(len(layers), 2)
@@ -553,10 +552,12 @@ class TestLayerObserver(common.TestCase):
         self.layer.add_clip(clip2)
         self.assertEqual(len(self.layer.get_clips()), 3)
 
-        with self.action_log.started("move clip"):
+        with self.action_log.started("move clip",
+                                     finalizing_action=CommitTimelineFinalizingAction(self.project.pipeline)):
             clip2.set_start(20 * Gst.SECOND)
         self.assertEqual(clip2.get_start(), 20 * Gst.SECOND)
-        self.assertEqual(len(self.layer.get_clips()), 2)
+        self.assertEqual(len(self.layer.get_clips()), 2,
+                         "The two title clips don't overlap so there should be no transition clip")
 
         self.action_log.undo()
         self.assertEqual(clip2.get_start(), 5 * Gst.SECOND)
@@ -566,6 +567,69 @@ class TestLayerObserver(common.TestCase):
         self.assertEqual(clip2.get_start(), 20 * Gst.SECOND)
         self.assertEqual(len(self.layer.get_clips()), 2)
 
+    @common.setup_project(assets_names=["mp3_sample.mp3"])
+    def test_move_transition_to_different_layer_audio(self):
+        uri = common.get_sample_uri("mp3_sample.mp3")
+        asset = GES.UriClipAsset.request_sync(uri)
+        self.__check_move_transition_to_different_layer(asset)
+
+    @common.setup_project(assets_names=["30fps_numeroted_frames_red.mkv"])
+    def test_move_transition_to_different_layer_video(self):
+        uri = common.get_sample_uri("30fps_numeroted_frames_red.mkv")
+        asset = GES.UriClipAsset.request_sync(uri)
+        self.__check_move_transition_to_different_layer(asset,
+                                                        border=123,
+                                                        invert=True,
+                                                        transition_type=GES.VideoStandardTransitionType.BAR_WIPE_LR)
+
+    def __check_move_transition_to_different_layer(self, asset, **props):
+        clip1 = asset.extract()
+        clip1.set_start(0 * Gst.SECOND)
+        self.layer.add_clip(clip1)
+        self.assertEqual(len(self.layer.get_clips()), 1)
+
+        clip2 = asset.extract()
+        clip2.set_start(clip1.props.duration * 9 // 10)
+        self.layer.add_clip(clip2)
+        clips = self.layer.get_clips()
+        self.assertEqual(len(clips), 3)
+
+        # Click all three clips including the transition clip to make sure
+        # it's not included in the group.
+        for clip in clips:
+            self.click_clip(clip, expect_selected=True, ctrl_key=True)
+        self.timeline_container.group_action.activate()
+
+        track_element = self.get_transition_element(self.layer)
+        for name, value in props.items():
+            track_element.set_property(name, value)
+
+        layer2 = self.timeline.append_layer()
+        with self.action_log.started("move clips to different layer"):
+            editing_context = EditingContext(clip1, self.timeline, GES.EditMode.EDIT_NORMAL, GES.Edge.EDGE_NONE, self.app)
+            editing_context.edit_to(0, layer2)
+            editing_context.finish()
+        self.assertEqual(len(self.layer.get_clips()), 0)
+        self.assertEqual(len(layer2.get_clips()), 3)
+        self.__check_transition_element(layer2, props)
+
+        with self.project.pipeline.commit_timeline_after():
+            self.action_log.undo()
+        self.assertEqual(len(self.layer.get_clips()), 3)
+        self.assertEqual(len(layer2.get_clips()), 0)
+        self.__check_transition_element(self.layer, props)
+
+        with self.project.pipeline.commit_timeline_after():
+            self.action_log.redo()
+        self.assertEqual(len(self.layer.get_clips()), 0)
+        self.assertEqual(len(layer2.get_clips()), 3)
+        self.__check_transition_element(layer2, props)
+
+    def __check_transition_element(self, layer: GES.Layer, props):
+        track_element = self.get_transition_element(layer)
+        for name, value in props.items():
+            self.assertEqual(track_element.get_property(name), value)
+
     @common.setup_timeline
     def test_transition_type(self):
         """Checks the transitions keep their type."""
@@ -698,11 +762,11 @@ class TestLayerObserver(common.TestCase):
             clip.ui.timeline._button_press_event_cb(None, event)
         clip.ui._button_release_event_cb(None, event)
 
-        self.timeline_container.copy_action.emit("activate", None)
+        self.timeline_container.copy_action.activate()
 
         position = 10
         project.pipeline.get_position = mock.Mock(return_value=position)
-        self.timeline_container.paste_action.emit("activate", None)
+        self.timeline_container.paste_action.activate()
         self.assertEqual(len(self.layer.get_clips()), 2)
 
         self.action_log.undo()
